package com.starbuds.server.service.media.dao;

import com.starbuds.server.common.pojo.db.AnchorDO;
import com.starbuds.server.common.pojo.media.LiveItemVO;
import com.starbuds.server.common.pojo.media.VipSearchDTO;
import com.starbuds.server.common.pojo.user.SingleChatUserVO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * 备注
 *
 * @author Lay
 * @date 2020/2/6
 */
@Mapper
@Repository
public interface AnchorDAO {
    /**
     * 查询主播直播列表
     *
     * @param pageCursor
     * @param pageSize
     * @param categoryId
     * @param liveStatus
     * @return
     */
    List<LiveItemVO> selectLiveItems(@Param("pageCursor") int pageCursor, @Param("pageSize") int pageSize, @Param("categoryId") Long categoryId, @Param("liveStatus") Integer liveStatus);

    /**
     * 统计主播直播列表
     *
     * @param categoryId
     * @param liveStatus
     * @return
     */
    int countLiveItems(@Param("categoryId") Long categoryId, @Param("liveStatus") Integer liveStatus);

    /**
     * 根据主播 id 列表查询
     *
     * @param userId
     * @return
     */
    List<LiveItemVO> selectLiveItemsByUserList(List<Long> userId);

    /**
     * 修改单聊状态
     *
     * @param userId
     * @param status
     * @param whereStatus
     * @return
     */
    int updateSingleChatStatus(@Param("userId") Long userId, @Param("status") Integer status, @Param("whereStatus") Integer whereStatus);

    /**
     * 查询单聊主播列表
     *
     * @param pageCursor
     * @param pageSize
     * @param singleChatStatus
     * @param userSex
     * @param categoryId
     * @param isOnline
     * @param singleChatCategories
     * @return
     */
    List<SingleChatUserVO> selectSingleChatUserList(@Param("pageCursor") int pageCursor,
                                                    @Param("pageSize") int pageSize,
                                                    @Param("singleChatStatus") Integer singleChatStatus,
                                                    @Param("userSex") Integer userSex,
                                                    @Param("categoryId") Long categoryId,
                                                    @Param("isOnline") Integer isOnline,
                                                    @Param("singleChatCategories") String singleChatCategories);

    /**
     * 根据 id 列表查询单聊主播列表
     *
     * @param list
     * @return
     */
    List<SingleChatUserVO> selectSingleChatUserListByIds(List<Long> list);

    /**
     * 根据 id 查询
     *
     * @param userId
     * @return
     */
    SingleChatUserVO selectSingleChatUserById(Long userId);

    /**
     * 根据 id 查询
     *
     * @param userId
     * @return
     */
    AnchorDO selectById(@Param("userId") Long userId);

    /**
     * 统计单聊主播
     *
     * @param singleChatStatus
     * @param userSex
     * @param categoryId
     * @param isOnline
     * @param singleChatCategories
     * @return
     */
    int countSingleChatUsers(@Param("singleChatStatus") Integer singleChatStatus,
                             @Param("userSex") Integer userSex,
                             @Param("categoryId") Long categoryId,
                             @Param("isOnline") Integer isOnline,
                             @Param("singleChatCategories") String singleChatCategories);

    /**
     * 增加超时次数
     *
     * @param userId
     * @param times
     * @return
     */
    int increaseSingleChatTimeoutTimes(@Param("userId") Long userId, @Param("times") int times);

    /**
     * 增加接通次数
     *
     * @param userId
     * @param times
     * @return
     */
    int increaseSingleChatConnectTimes(@Param("userId") Long userId, @Param("times") int times);

    /**
     * 增加单聊分钟数
     *
     * @param userId
     * @param minutes
     * @return
     */
    int increaseSingleChatMinutes(@Param("userId") Long userId, @Param("minutes") long minutes);

    /**
     * 增加单聊评分
     *
     * @param userId
     * @param score
     * @return
     */
    int increaseSingleChatScore(@Param("userId") Long userId, @Param("score") long score);

    /**
     * 增加单聊评价次数
     *
     * @param userId
     * @param times
     * @return
     */
    int increaseSingleChatEvaluateTimes(@Param("userId") Long userId, @Param("times") int times);

    /**
     * 根据 id 统计
     *
     * @param userId
     * @return
     */
    int countById(@Param("userId") Long userId);

    /**
     * 根据 id 统计单聊主播
     *
     * @param userId
     * @return
     */
    int countSingleChatMathEnableById(@Param("userId") Long userId);

    /**
     * 查询机器人主播
     *
     * @return
     */
    List<Long> selectSingleChatRobotAnchorIds();

    /**
     * 获取vip搜索列表
     *
     * @param searchDTO
     * @return
     */
    List<SingleChatUserVO> selectVipSingleChatList(VipSearchDTO searchDTO);

    /**
     * 获取vip搜索列表大小
     *
     * @param searchDTO
     * @return
     */
    int countVipSingleChatList(VipSearchDTO searchDTO);

    /**
     * 更新 pk 状态
     *
     * @param userId
     * @param battling
     * @param whereBattling
     * @return
     */
    int updateBattling(@Param("userId") Long userId, @Param("battling") Integer battling, @Param("whereBattling") Integer whereBattling);

    /**
     * 增加 PK 次数
     *
     * @param userId
     * @param delta
     * @return
     */
    int incrBattleTimes(@Param("userId") Long userId, @Param("delta") int delta);

    /**
     * 增加 PK 胜场
     *
     * @param userId
     * @param delta
     * @return
     */
    int incrBattleWinTimes(@Param("userId") Long userId, @Param("delta") int delta);

    /**
     * 增加 PK 输场
     *
     * @param userId
     * @param delta
     * @return
     */
    int incrBattleLossTimes(@Param("userId") Long userId, @Param("delta") int delta);

    /**
     * 增加平局场次
     *
     * @param userIds
     * @param delta
     * @return
     */
    int incrBattleDrawTimes(@Param("userIds") List<Long> userIds, @Param("delta") int delta);

    /**
     * 查询 PK 状态
     *
     * @param userId
     * @return
     */
    Integer selectBattling(@Param("userId") Long userId);
}
