package com.starbuds.server.service.media.dao;

import com.starbuds.server.common.pojo.db.GiftDO;
import com.starbuds.server.common.pojo.db.LotteryGameWinHisDO;
import com.starbuds.server.common.pojo.media.GiftPayItem;
import com.starbuds.server.common.pojo.user.UserGiftWallItem;
import org.apache.ibatis.annotations.MapKey;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * 备注
 *
 * @author Lay
 * @date 2020/1/2
 */
@Mapper
@Repository
public interface GiftDAO {
    /**
     * 获取礼物列表
     *
     * @param giftType
     * @param giftCategory
     * @param giftPriceType
     * @param curTime
     * @return
     */
    List<GiftDO> selectAll(@Param("giftType") Integer giftType, @Param("giftCategory") Integer giftCategory, @Param("giftPriceType") Integer giftPriceType, @Param("displayType") int displayType, @Param("curTime") long curTime);

    /**
     * 根据 id 列表查询礼物列表
     *
     * @param giftId
     * @return
     */
    List<GiftDO> selectAllByIds(List<Long> giftId);

    /**
     * 根据 id 和当前时间统计
     *
     * @param giftId
     * @param curTime
     * @return
     */
    int countByIdAndTime(@Param("giftId") Long giftId, @Param("curTime") long curTime);

    /**
     * 查询支付信息
     *
     * @param giftId
     * @return
     */
    GiftPayItem selectPayItem(@Param("giftId") Long giftId);

    /**
     * 查询财富值
     *
     * @param giftId
     * @return
     */
    Long selectWealthValue(@Param("giftId") Long giftId);

    /**
     * 根据 id 查询
     *
     * @param giftId
     * @param status
     * @return
     */
    GiftDO selectById(@Param("giftId") Long giftId, @Param("status") Integer status);

    /**
     * 查询礼物墙
     *
     * @param giftIds
     * @return
     */
    List<UserGiftWallItem> selectWallItems(List<Long> giftIds);

    /**
     * 统计礼物墙礼物数量
     *
     * @return
     */
    int countByGiftWall();

    /**
     * 通过lotteryId获取列表
     *
     * @param lotteryIds
     * @return
     */
    List<GiftDO> getGiftListByLotteryIds(@Param("lotteryIds") List<String> lotteryIds);

    /**
     * 通过Id获取列表
     *
     * @param giftIds
     * @return
     */
    List<GiftDO> getGiftListByGiftIds(@Param("giftIds") List<Long> giftIds);

    /**
     * 获得打赏礼物列表
     *
     * @param displayType
     * @return
     */
    List<GiftDO> selectRewardGifts(@Param("displayType") String displayType);

    /**
     * 查询打赏礼物数量
     *
     * @param displayType
     * @param giftId
     * @return
     */
    int countRewardGift(@Param("displayType") String displayType, @Param("giftId") Long giftId);

    /**
     * 根据ids获得 DO Map
     *
     * @param ids id集合
     * @return Map<主键, DO对象>
     */
    @MapKey("giftId")
    Map<Long, GiftDO> selectPriceMapByIds(@Param("ids") Collection ids);
}
