package com.starbuds.server.service.media.dao;

import com.starbuds.server.common.pojo.db.LiveHisDO;
import com.starbuds.server.common.pojo.media.LiveHisVO;
import com.starbuds.server.common.pojo.media.LiveItemVO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * 备注
 *
 * @author Lay
 * @date 2019/12/23
 */
@Mapper
@Repository
public interface LiveHisDAO {
    /**
     * 创建直播记录
     *
     * @param liveHisDO
     * @return
     */
    int insert(LiveHisDO liveHisDO);

    /**
     * 更新直播记录
     *
     * @param liveHisDO
     * @return
     */
    int update(LiveHisDO liveHisDO);

    /**
     * 根据 id 查询
     *
     * @param hisId
     * @return
     */
    LiveHisDO selectById(@Param("hisId") Long hisId);

    /**
     * 更新有效时长
     *
     * @param hisId
     * @param liveEffectiveDuration
     * @return
     */
    int updateEffectiveDurationById(@Param("hisId") Long hisId, @Param("liveEffectiveDuration") Long liveEffectiveDuration);

    /**
     * 根据 id 统计
     *
     * @param hisId
     * @param userId
     * @return
     */
    int countById(@Param("hisId") Long hisId, @Param("userId") Long userId);

    /**
     * 查询直播列表
     *
     * @param pageCursor
     * @param pageSize
     * @param categoryId
     * @param liveStatus
     * @return
     */
    List<LiveItemVO> selectAllItems(@Param("pageCursor") int pageCursor, @Param("pageSize") int pageSize, @Param("categoryId") Long categoryId, @Param("liveStatus") Integer liveStatus);

    /**
     * 统计直播列表
     *
     * @param categoryId
     * @param liveStatus
     * @return
     */
    int countItems(@Param("categoryId") Long categoryId, @Param("liveStatus") Integer liveStatus);

    /**
     * 查询关注直播列表
     *
     * @param pageCursor
     * @param pageSize
     * @param userId
     * @param liveStatus
     * @return
     */
    List<LiveItemVO> selectAllFollowItems(@Param("pageCursor") int pageCursor, @Param("pageSize") int pageSize, @Param("userId") Long userId, @Param("liveStatus") Integer liveStatus);

    /**
     * 统计关注的直播列表
     *
     * @param userId
     * @param liveStatus
     * @return
     */
    int countFollowItems(@Param("userId") Long userId, @Param("liveStatus") Integer liveStatus);

    /**
     * 根据时间统计
     *
     * @param userId
     * @param beginTime
     * @param endTime
     * @return
     */
    int countByTime(@Param("userId") Long userId, @Param("beginTime") long beginTime, @Param("endTime") long endTime);

    /**
     * 查询直播记录列表
     *
     * @param pageCursor
     * @param pageSize
     * @param userId
     * @param beginTime
     * @param endTime
     * @return
     */
    List<LiveHisVO> selectAll(@Param("pageCursor") int pageCursor, @Param("pageSize") int pageSize, @Param("userId") Long userId, @Param("beginTime") Long beginTime, @Param("endTime") Long endTime);

    /**
     * 统计直播记录列表
     *
     * @param userId
     * @param beginTime
     * @param endTime
     * @return
     */
    int countAll(@Param("userId") Long userId, @Param("beginTime") Long beginTime, @Param("endTime") Long endTime);
}
