package com.starbuds.server.service.media.dao;

import com.starbuds.server.common.pojo.db.SingleChatHisDO;
import com.starbuds.server.common.pojo.media.SingleChatMatchedHis;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * 备注
 *
 * @author Lay
 * @date 2020/3/22
 */
@Mapper
@Repository
public interface SingleChatHisDAO {
    /**
     * 创建记录
     *
     * @param singleChatHisDO
     * @return
     */
    int insert(SingleChatHisDO singleChatHisDO);

    /**
     * 查询状态
     *
     * @param hisId
     * @param userId
     * @param targetUserId
     * @return
     */
    Integer selectStatus(@Param("hisId") String hisId,
                         @Param("userId") Long userId,
                         @Param("targetUserId") Long targetUserId);

    /**
     * 根据 id 查询
     *
     * @param hisId
     * @return
     */
    SingleChatHisDO selectById(@Param("hisId") String hisId);

    /**
     * 更新状态
     *
     * @param hisId
     * @param status
     * @param whereStatus
     * @param updateTime
     * @return
     */
    int updateStatus(@Param("hisId") String hisId,
                     @Param("status") Integer status,
                     @Param("whereStatus") Integer whereStatus,
                     @Param("updateTime") Long updateTime);

    /**
     * 更新状态和分钟数
     *
     * @param hisId
     * @param status
     * @param whereStatus
     * @param updateTime
     * @param hisMinutes
     * @return
     */
    int updateStatusWithMinutes(@Param("hisId") String hisId,
                                @Param("status") Integer status,
                                @Param("whereStatus") Integer whereStatus,
                                @Param("updateTime") Long updateTime,
                                @Param("hisMinutes") Integer hisMinutes,
                                @Param("hisCoinMinutes") Integer hisCoinMinutes,
                                @Param("hisTimeMinutes") Integer hisTimeMinutes);

    /**
     * 查询匹配记录
     *
     * @param num
     * @param hisStatusList
     * @return
     */
    List<SingleChatMatchedHis> selectMatchedHis(@Param("num") int num, @Param("hisStatusList") List<Integer> hisStatusList);

    /**
     * 更新回放地址
     *
     * @param hisId
     * @param hisReplay
     * @return
     */
    int updateReplay(@Param("hisId") String hisId, @Param("hisReplay") String hisReplay);

    /**
     * 更新回放 id
     *
     * @param hisId
     * @param hisReplayId
     * @param resourceId
     * @return
     */
    int updateReplayId(@Param("hisId") String hisId,
                       @Param("hisReplayId") String hisReplayId,
                       @Param("resourceId") String resourceId);
}
