package com.starbuds.server.service.media.dao;

import com.starbuds.server.common.pojo.db.VoiceChatHisDO;
import com.starbuds.server.common.pojo.media.VoiceChatDTO;
import com.starbuds.server.common.pojo.user.UserMatchVO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/11/16 13:01
 * @Description 语音速配持久层
 */
@Mapper
@Repository
public interface VoiceChatHisDAO {

    /**
     * 插入
     *
     * @param chatHisDO
     * @return
     */
    int insert(VoiceChatHisDO chatHisDO);

    /**
     * 获取语音速配记录
     *
     * @param hisId
     * @return
     */
    VoiceChatHisDO selectById(@Param("hisId") Long hisId);

    /**
     * 更新
     *
     * @param chatHisUpdate
     * @return
     */
    int updateById(VoiceChatHisDO chatHisUpdate);

    /**
     * 更新录播地址
     *
     * @param hisId
     * @param hisReplay
     * @return
     */
    int updateReplay(@Param("hisId") Long hisId, @Param("hisReplay") String hisReplay);

    /**
     * 更新录播id
     *
     * @param hisId
     * @param hisReplayId
     * @param hisResourceId
     * @return
     */
    int updateReplayId(@Param("hisId") Long hisId, @Param("hisReplayId") String hisReplayId, @Param("hisResourceId") String hisResourceId);

    /**
     * 更新
     *
     * @param hisDO
     * @param whereHisStatus
     * @return
     */
    int updateByIdAndWhereStatus(@Param("hisDO") VoiceChatHisDO hisDO, @Param("whereHisStatus") Integer whereHisStatus);

    /**
     * 更新状态和是分钟
     *
     * @param hisId
     * @param hisStatus
     * @param whereHisStatus
     * @param hisStopTime
     * @param hisUpdateTime
     * @param hisMinutes
     * @param stopReason
     * @return
     */
    int updateStatusWithMinutes(@Param("hisId") Long hisId, @Param("hisStatus") Integer hisStatus, @Param("whereHisStatus") Integer whereHisStatus, @Param("hisStopTime") Long hisStopTime, @Param("hisUpdateTime") Long hisUpdateTime, @Param("hisMinutes") Integer hisMinutes, @Param("stopReason") Integer stopReason);

    /**
     * 获取记录状态
     *
     * @param hisId
     * @return
     */
    Integer selectHisStatus(@Param("hisId") Long hisId);

    /**
     * 获取语音速配列表
     *
     * @param pageCursor
     * @param pageSize
     * @param hisStatus
     * @param userId
     * @return
     */
    List<VoiceChatDTO> selectVoiceChatList(@Param("pageCursor") int pageCursor, @Param("pageSize") int pageSize, @Param("hisStatus") Integer hisStatus, @Param("userId") Long userId);

    /**
     * 获取语音速配列表大小
     *
     * @param hisStatus
     * @param userId
     * @return
     */
    int countVoiceChatList(@Param("hisStatus") Integer hisStatus, @Param("userId") Long userId);

    /**
     * 判断是否存在
     *
     * @param hisId
     * @return
     */
    int countById(@Param("hisId") Long hisId);

    /**
     * 更新匹配到的用户
     *
     * @param hisId
     * @param matchUserIds
     */
    int updateMatchUserIds(@Param("hisId") Long hisId, @Param("matchUserIds") String matchUserIds);

    /**
     * 获取发起人
     *
     * @param hisId
     * @return
     */
    Long selectUserId(@Param("hisId") Long hisId);

    /**
     * 获取记录id
     *
     * @param userId
     * @return
     */
    Long selectHisId(@Param("userId") Long userId);

    /**
     * 获取匹配成功记录
     *
     * @return
     */
    List<UserMatchVO> selectMatchSuccessList();
}
