package com.starbuds.server.service.media.impl;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.finance.WalletTypeEnum;
import com.starbuds.server.common.constant.message.im.ImMsgType;
import com.starbuds.server.common.constant.message.im.ImTargetType;
import com.starbuds.server.common.pojo.db.ChatGroupDO;
import com.starbuds.server.common.pojo.db.ImGroupChatMsgDO;
import com.starbuds.server.common.pojo.message.im.chat.HeadlineMsg;
import com.starbuds.server.common.pojo.message.im.chat.ImGroupChatMsgVO;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.finance.WalletService;
import com.starbuds.server.common.service.media.ChatService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.service.media.dao.ChatGroupDAO;
import com.starbuds.server.service.media.dao.ImGroupChatMsgDAO;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;

/**
 * 聊天服务实现
 *
 * @author Lay
 * @date 2020/3/23
 */
@Service(version = "1.0.0")
public class ChatServiceImpl implements ChatService {

    private static final String CHAT_GROUP_ID = "BCR2020";

    @Resource
    ChatGroupDAO chatGroupDAO;

    @Resource
    ImGroupChatMsgDAO imGroupChatMsgDAO;

    @Reference(version = "1.0.0")
    ImService imService;

    @Reference(version = "1.0.0")
    WalletService walletService;

    @Reference(version = "1.0.0")
    IdentityService identityService;

    /**
     * 获取聊天组
     *
     * @param groupId
     * @return
     */
    @Override
    public ChatGroupDO getChatGroup(String groupId) {
        return chatGroupDAO.selectById(groupId);
    }

    /**
     * 发送头条消息
     *
     * @param userId
     * @param msgContent
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean sendHeadline(Long userId, String msgContent) {
        return sendHeadline(userId, msgContent, false);
    }

    /**
     * 发送头条消息
     *
     * @param userId
     * @param msgContent
     * @param isFree
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean sendHeadline(Long userId, String msgContent, boolean isFree) {
        // 判断余额
        if (walletService.getBalance(userId, WalletTypeEnum.STAR_DIAMOND) < 100L) {
            return false;
        }

        Long msgId = identityService.getId();
        Long timestamp = System.currentTimeMillis();

        HeadlineMsg msg = new HeadlineMsg();
        msg.setMsgContent(msgContent);
        msg.setGroupId("0");
        msg.setSenderId(userId);
        msg.setMsgId(msgId);
        msg.setTargetType(ImTargetType.GROUP);

        // 创建记录
        ImGroupChatMsgDO chatMsgDO = new ImGroupChatMsgDO();
        chatMsgDO.setMsgId(msgId);
        chatMsgDO.setGroupId("0");
        chatMsgDO.setSenderId(userId);
        chatMsgDO.setMsgType(msg.getType());
        chatMsgDO.setMsgContent(msgContent);
        chatMsgDO.setMsgData(JSON.toJSONString(msg));
        chatMsgDO.setTimestamp(timestamp);
        imGroupChatMsgDAO.insert(chatMsgDO);

        // 扣款
        if (!isFree) {
            Long balance = walletService.reduceCoin(userId, WalletTypeEnum.STAR_DIAMOND.getValue(), 1000L, "抢头条", msgId);
            if (balance == null) {
                throw new RuntimeException("余额变动失败");
            } else if (balance < 0) {
                throw new RuntimeException("余额不足");
            }

            // 发送群组消息
            if (!imService.sendRtcRoomMsg(ImTargetType.GROUP, userId, null, 0L, msg)) {
                throw new RuntimeException("发送失败");
            }
        }

        return true;
    }

    /**
     * 获取头条
     *
     * @return
     */
    @Override
    public ImGroupChatMsgVO getHeadline() {
        return imGroupChatMsgDAO.selectLast(ImMsgType.HEADLINE_MSG_TOP);
    }
}
