package com.starbuds.server.service.media.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.cache.RedisCacheKeys;
import com.starbuds.server.common.constant.common.CommonStatus;
import com.starbuds.server.common.constant.finance.WalletTypeEnum;
import com.starbuds.server.common.constant.gift.GiftDisplayTypeEnum;
import com.starbuds.server.common.constant.user.NobleLevelEnum;
import com.starbuds.server.common.pojo.activity.GiftLoveItem;
import com.starbuds.server.common.pojo.activity.GiftWeekItem;
import com.starbuds.server.common.pojo.db.GiftComboConfigDO;
import com.starbuds.server.common.pojo.db.GiftDO;
import com.starbuds.server.common.pojo.media.GiftItemVO;
import com.starbuds.server.common.pojo.media.GiftItemsGetQuery;
import com.starbuds.server.common.pojo.media.GiftPayItem;
import com.starbuds.server.common.pojo.media.bo.PriceInfoBO;
import com.starbuds.server.common.pojo.user.UserGiftWallItem;
import com.starbuds.server.common.pojo.user.WeekUser;
import com.starbuds.server.common.service.activity.ActivityService;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.finance.WalletService;
import com.starbuds.server.common.service.media.GiftService;
import com.starbuds.server.common.service.user.AnchorService;
import com.starbuds.server.common.service.user.NobleService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.service.media.dao.GiftComboConfigDAO;
import com.starbuds.server.service.media.dao.GiftDAO;
import com.starbuds.server.service.media.dao.GiftLoveDAO;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.beans.BeanUtils;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 礼物服务实现
 *
 * @author Lay
 * @date 2020/1/2
 */
@Service(version = "1.0.0")
public class GiftServiceImpl implements GiftService {

    @Resource
    GiftDAO giftDAO;

    @Resource
    GiftComboConfigDAO giftComboConfigDAO;

    @Resource
    GiftLoveDAO giftLoveDAO;

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    AnchorService anchorService;

    @Reference(version = "1.0.0")
    WalletService walletService;

    @Reference(version = "1.0.0")
    NobleService nobleService;

    @Reference(version = "1.0.0")
    ActivityService activityService;

    @Reference(version = "1.0.0")
    RedisService redisService;

    /**
     * 获取礼物列表
     *
     * @param query
     * @return
     */
    @Override
    public List<GiftItemVO> getGiftItems(GiftItemsGetQuery query) {
        Map<Integer, String> walletTypeNameMap = walletService.getWalletTypeNameMap();
        List<GiftDO> giftDOList = giftDAO.selectAll(query.getGiftType(), query.getGiftCategory(), query.getGiftPriceType(), query.getDisplayType(), System.currentTimeMillis());
        List<GiftItemVO> giftItemVOList = new ArrayList<>();
        if (CollectionUtil.isNotEmpty(giftDOList)) {

            // 获取周星礼物
            List<GiftWeekItem> weekGifts = activityService.getWeekGifts();
            List<GiftWeekItem> weekGiftsPrevious = activityService.getWeekGiftsPrevious();
            // 周星标签
            Long tagId = activityService.getTagId();

            // 本周周星礼物
            List<Long> weekGiftIds = weekGifts.stream().map(GiftWeekItem::getGiftId).collect(Collectors.toList());
            // 上周周星礼物
            Map<Long, GiftWeekItem> giftWeekItemMap = weekGiftsPrevious.stream().collect(Collectors.toMap(GiftWeekItem::getGiftId, Function.identity()));

            for (GiftDO giftDO : giftDOList) {
                GiftItemVO giftItemVO = new GiftItemVO();
                BeanUtils.copyProperties(giftDO, giftItemVO);
                giftItemVO.setGiftPriceTypeName(walletTypeNameMap.get(giftItemVO.getGiftPriceType()));

                if (weekGiftIds.contains(giftDO.getGiftId())) {
                    giftItemVO.setWeekStar(true);
                    giftItemVO.setTagId(tagId);
                }

                if (giftWeekItemMap.keySet().contains(giftDO.getGiftId())) {
                    giftItemVO.setLastWeekStar(true);
                    // 壕星榜首
                    WeekUser richTopUser = giftWeekItemMap.get(giftDO.getGiftId()).getRichTopUser();
                    giftItemVO.setNamingUserId(Optional.ofNullable(richTopUser).map(WeekUser::getUserId).orElse(null));
                    giftItemVO.setNamingUserName(Optional.ofNullable(richTopUser).map(WeekUser::getUserName).orElse(null));
                }
                giftItemVOList.add(giftItemVO);
            }
        }
        return giftItemVOList;
    }

    /**
     * 获取礼物
     *
     * @param giftId
     * @return
     */
    @Override
    public GiftItemVO getGift(Long giftId) {
        Map<Integer, String> walletTypeNameMap = walletService.getWalletTypeNameMap();
        GiftDO giftDO = giftDAO.selectById(giftId, CommonStatus.Normal);
        if (giftDO != null) {
            GiftItemVO giftItemVO = new GiftItemVO();
            BeanUtils.copyProperties(giftDO, giftItemVO);
            giftItemVO.setGiftPriceTypeName(walletTypeNameMap.get(giftItemVO.getGiftPriceType()));
            return giftItemVO;
        }
        return null;
    }

    /**
     * 判断礼物是否有效
     *
     * @param giftId
     * @return
     */
    @Override
    public boolean isGiftEnabled(Long giftId) {
        return giftDAO.countByIdAndTime(giftId, System.currentTimeMillis()) > 0;
    }

    /**
     * 获取礼物支付信息
     *
     * @param giftId
     * @return
     */
    @Override
    public GiftPayItem getPayItem(Long giftId) {
        return giftDAO.selectPayItem(giftId);
    }

    /**
     * 查询礼物
     *
     * @param giftId
     * @param status
     * @return
     */
    @Override
    public GiftDO getGift(Long giftId, Integer status) {
        return giftDAO.selectById(giftId, status);
    }

    /**
     * 根据 id 列表获取礼物列表
     *
     * @param giftIds
     * @return
     */
    @Override
    public List<GiftItemVO> getGiftItemsByIds(List<Long> giftIds) {
        if (CollectionUtils.isEmpty(giftIds)) {
            return Collections.emptyList();
        }
        Map<Integer, String> walletTypeNameMap = walletService.getWalletTypeNameMap();
        List<GiftDO> giftDOList = giftDAO.selectAllByIds(giftIds);
        if (CollectionUtil.isNotEmpty(giftDOList)) {
            List<GiftItemVO> list = new ArrayList<>();
            for (GiftDO giftDO : giftDOList) {
                GiftItemVO giftItemVO = new GiftItemVO();
                BeanUtils.copyProperties(giftDO, giftItemVO);
                giftItemVO.setGiftPriceTypeName(walletTypeNameMap.get(giftItemVO.getGiftPriceType()));
                list.add(giftItemVO);
            }
            return list;
        }
        return Collections.emptyList();
    }

    /**
     * 礼物条件判断
     *
     * @param giftId
     * @param anchorUserId
     * @param userId
     * @return
     */
    @Override
    public String checkGiftConditions(Long giftId, Long anchorUserId, Long userId) {
        GiftDO giftDO = giftDAO.selectById(giftId, null);

        // 用户等级限制
        Integer wealthLevelLimit = Optional.ofNullable(giftDO.getWealthLevelLimit()).orElse(0);
        if (wealthLevelLimit > 0) {
            if (wealthLevelLimit > userService.getWealthLevelByUserId(userId)) {
                return String.format("等级达到%d级才可以赠送哦，加油升级吧！", wealthLevelLimit);
            }
        }

        // 贵族等级限制
        Integer nobleLevelLimit = Optional.ofNullable(giftDO.getNobleLevelLimit()).orElse(0);
        if (nobleLevelLimit > 0) {
            if (nobleLevelLimit > nobleService.getUserNobleLevel(userId)) {
                return String.format("%s贵族及以上才可赠送哦，快去开通贵族吧！", Optional.ofNullable(NobleLevelEnum.toEnum(nobleLevelLimit)).map(NobleLevelEnum::getDesc).orElse(""));
            }
        }

        // 粉丝团限制
        if (anchorUserId != null) {
            Integer fansLevelLimit = Optional.ofNullable(giftDO.getFansLevelLimit()).orElse(0);
            if (fansLevelLimit > 0) {
                // 判断是否粉丝团成员
                if (!anchorService.isFans(anchorUserId, userId)) {
                    return String.format("加入主播粉丝团才可赠送哦，赶快加入吧！");
                }
            }
        }

        return null;
    }

    /**
     * 获取礼物墙礼物
     *
     * @param giftIds
     * @return
     */
    @Override
    public List<UserGiftWallItem> getWallItemsByIds(List<Long> giftIds) {
        return giftDAO.selectWallItems(giftIds);
    }

    /**
     * 获取礼物墙礼物数量
     *
     * @return
     */
    @Override
    public int getWallItemsCount() {
        return giftDAO.countByGiftWall();
    }

    /**
     * 获取礼物连击配置
     *
     * @param giftId
     * @return
     */
    @Override
    public GiftComboConfigDO getComboConfig(long giftId) {
        return giftComboConfigDAO.selectLatestEnabled();
    }

    /**
     * 获取缓存的连击配置
     *
     * @param giftId
     * @return
     */
    @Override
    public GiftComboConfigDO getCachedComboConfig(long giftId) {
        String config = redisService.get(RedisCacheKeys.COMBO_CONFIG);
        try {
            if (StringUtils.isNotBlank(config)) {
                return JSON.parseObject(config, GiftComboConfigDO.class);
            }
        } catch (Exception ignored) {
        }

        GiftComboConfigDO comboConfigDO = getComboConfig(giftId);
        if (comboConfigDO != null) {
            // 写入缓存
            redisService.set(RedisCacheKeys.COMBO_CONFIG, JSON.toJSONString(comboConfigDO), 60, TimeUnit.SECONDS);
        }
        return comboConfigDO;
    }

    /**
     * 判断是否真爱榜礼物
     *
     * @param giftId
     * @return
     */
    @Override
    public boolean isTrueLoveGift(Long giftId) {
        return giftLoveDAO.countById(giftId) > 0;
    }

    /**
     * 获取真爱榜礼物
     *
     * @return
     */
    @Override
    public List<GiftLoveItem> getTrueLoveGifts() {
        return giftLoveDAO.selectTrueLoveGifts();
    }


    /**
     * 通过活动获取礼物
     *
     * @param lotteryIds
     * @return
     */
    @Override
    public Map<String, GiftDO> getGiftMapByLotteryIds(List<String> lotteryIds) {
        if (CollectionUtil.isNotEmpty(lotteryIds)) {
            Map<String, GiftDO> res = new HashMap<>(lotteryIds.size());
            List<GiftDO> giftDOList = giftDAO.getGiftListByLotteryIds(lotteryIds);
            giftDOList.forEach(i -> res.put(i.getLotteryId(), i));
            return res;
        }
        return new HashMap<>();
    }

    /**
     * 通过Id获取礼物
     *
     * @param giftIds
     * @return
     */
    @Override
    public Map<Long, GiftDO> getGiftMapByGiftIds(List<Long> giftIds) {
        if (CollectionUtil.isNotEmpty(giftIds)) {
            Map<Long, GiftDO> res = new HashMap<>(giftIds.size());
            List<GiftDO> giftDOList = giftDAO.getGiftListByGiftIds(giftIds);
            giftDOList.forEach(i -> res.put(i.getGiftId(), i));
            return res;
        }
        return new HashMap<>();
    }

    /**
     * 获得打赏礼物列表
     *
     * @return
     */
    @Override
    public List<GiftDO> getRewardGifts() {
        List<GiftDO> list = giftDAO.selectRewardGifts(GiftDisplayTypeEnum.REWARD.getValue() + "");
        if (CollectionUtils.isEmpty(list)) {
            return Collections.emptyList();
        }
        return list;
    }

    /**
     * 打赏礼物是否存在
     *
     * @param giftId
     * @return true 存在
     */
    @Override
    public boolean existRewardGift(Long giftId) {
        if (giftId == null) {
            return false;
        }
        return giftDAO.countRewardGift(GiftDisplayTypeEnum.REWARD.getValue() + "", giftId) > 0;
    }

    @Override
    public Map<Long, PriceInfoBO> getGiftPriceMap(Collection<Long> giftIds) {
        if (CollectionUtils.isEmpty(giftIds)) {
            return Collections.emptyMap();
        }
        List<Long> ids = giftIds.stream().filter(Objects::nonNull).distinct().collect(Collectors.toList());
        Map<Long, GiftDO> map = giftDAO.selectPriceMapByIds(ids);
        Map<Long, PriceInfoBO> priceMap = new HashMap<>();
        ids.stream()
                .forEach(giftId -> {
                    GiftDO gift = map.get(giftId);
                    if (gift == null) {
                        return;
                    }
                    PriceInfoBO bo = new PriceInfoBO();
                    bo.setPrice(gift.getGiftPrice());
                    bo.setType(WalletTypeEnum.toEnum(gift.getGiftPriceType()));
                    priceMap.put(giftId, bo);
                });
        return priceMap;
    }

    @Override
    public long getWealthValue(long giftId) {
        Long wealthValue = giftDAO.selectWealthValue(giftId);
        if (wealthValue == null) {
            return 0L;
        }
        return wealthValue;
    }
}
