package com.starbuds.server.service.media.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.crypto.SecureUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.server.common.constant.cache.RedisCacheKeys;
import com.starbuds.server.common.constant.cache.RedisChatKeys;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.common.TimeConstants;
import com.starbuds.server.common.constant.finance.ChatTimeWalletType;
import com.starbuds.server.common.constant.finance.WalletTypeEnum;
import com.starbuds.server.common.constant.media.ChatGroupType;
import com.starbuds.server.common.constant.media.ChatStatusEnum;
import com.starbuds.server.common.constant.media.LiveStatusEnum;
import com.starbuds.server.common.constant.media.SingleChatStopReasonEnum;
import com.starbuds.server.common.constant.message.im.ImTargetType;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.constant.statistics.StatPlatformTypeEnum;
import com.starbuds.server.common.constant.user.SingleChatStatus;
import com.starbuds.server.common.pojo.zego.RtcServiceCallback;
import com.starbuds.server.common.pojo.zego.ZegoServerCallbackType;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.data.NearUserDTO;
import com.starbuds.server.common.pojo.data.NearbyUser;
import com.starbuds.server.common.pojo.db.*;
import com.starbuds.server.common.pojo.media.FreeSingleChatMinutesDTO;
import com.starbuds.server.common.pojo.media.SingleChatMatchedHis;
import com.starbuds.server.common.pojo.media.SingleChatRequestDTO;
import com.starbuds.server.common.pojo.media.VipSearchDTO;
import com.starbuds.server.common.pojo.message.im.chat.*;
import com.starbuds.server.common.pojo.mq.QueueChatUserExitMsg;
import com.starbuds.server.common.pojo.mq.QueueChatUserJoinMsg;
import com.starbuds.server.common.pojo.mq.QueueSingleChatChargingTickMsg;
import com.starbuds.server.common.pojo.mq.QueueSingleChatRingTimeoutMsg;
import com.starbuds.server.common.pojo.rtc.RtcProviderSignature;
import com.starbuds.server.common.pojo.user.AnchorSingleChatInfoVO;
import com.starbuds.server.common.pojo.user.SingleChatUserVO;
import com.starbuds.server.common.service.core.CacheService;
import com.starbuds.server.common.service.data.UserSearchService;
import com.starbuds.server.common.service.finance.PayService;
import com.starbuds.server.common.service.finance.VipService;
import com.starbuds.server.common.service.finance.WalletService;
import com.starbuds.server.common.service.media.SingleChatService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.statistics.StatPlatformService;
import com.starbuds.server.common.service.user.AnchorService;
import com.starbuds.server.common.service.user.ChannelService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.service.media.agora.constant.AgoraRealTimeEventType;
import com.starbuds.server.service.media.dao.*;
import com.starbuds.server.service.media.provider.AgoraProvider;
import com.starbuds.server.service.media.provider.ChatMatchProvider;
import com.starbuds.server.service.media.provider.ProviderBuilder;
import com.starbuds.server.service.media.provider.SingleChatProvider;
import com.starbuds.server.service.media.rocket.RocketProducer;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 视频单聊服务实现
 *
 * @author Lay
 * @date 2020/3/22
 */
@Service(version = "1.0.0")
@Slf4j
public class SingleChatServiceImpl implements SingleChatService {

    @Reference(version = "1.0.0")
    CacheService cacheService;

    @Reference(version = "1.0.0")
    AnchorService anchorService;

    @Reference(version = "1.0.0")
    ImService imService;

    @Reference(version = "1.0.0")
    WalletService walletService;

    @Reference(version = "1.0.0")
    StatPlatformService statPlatformService;

    @Reference(version = "1.0.0")
    ChannelService channelService;

    @Reference(version = "1.0.0")
    PayService payService;

    @Reference(version = "1.0.0")
    UserSearchService userSearchService;

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    VipService vipService;

    @Resource
    RocketProducer rocketProducer;

    @Resource
    ProviderBuilder providerBuilder;


    @Resource
    SingleChatProvider singleChatProvider;

    @Resource
    ChatMatchProvider chatMatchProvider;

    @Resource
    AnchorDAO anchorDAO;

    @Resource
    SingleChatHisDAO singleChatHisDAO;

    @Resource
    SingleChatSuggestDAO singleChatSuggestDAO;

    @Resource
    ChatGroupDAO chatGroupDAO;

    @Resource
    SingleChatEvaluateDAO singleChatEvaluateDAO;

    @Resource
    UserSingleChatTagDAO userSingleChatTagDAO;

    @Resource
    SingleChatMatchHisDAO singleChatMatchHisDAO;

    @Resource
    SingleChatFreeHisDAO singleChatFreeHisDAO;

    /**
     * 发起单聊请求
     *
     * @param singleChatRequestDTO
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public String sendSingleChatRequest(SingleChatRequestDTO singleChatRequestDTO) {
        Long userId = singleChatRequestDTO.getUserId();
        Long targetUserId = singleChatRequestDTO.getTargetUserId();
        Long curTime = System.currentTimeMillis();

        AnchorSingleChatInfoVO singleChatInfo = anchorService.getSingleChatInfo(targetUserId);
        if (singleChatInfo == null) {
            return null;
        }


        // 如果是匹配锁定的
        if (cacheService.hasKey(RedisCacheKeys.getSingleChatMatchLock(targetUserId))) {
            return null;
        }

        // 开启用户占用
        String lockKey = RedisChatKeys.getSingleChatLock(targetUserId);
        try {
            if (cacheService.setIfAbsent(lockKey, String.valueOf(userId), TimeConstants.SECOND_30_MILLS, TimeUnit.MILLISECONDS)) {
                // 修改状态
                if (anchorDAO.updateSingleChatStatus(targetUserId, SingleChatStatus.BUSY, SingleChatStatus.IDLE) == 0) {
                    throw new RuntimeException("状态更改失败");
                }

                // 创建记录
                String hisId = SecureUtil.md5(String.format("%d:%d:%d", userId, targetUserId, curTime));
                SingleChatHisDO singleChatHis = new SingleChatHisDO();
                singleChatHis.setHisId(hisId);
                singleChatHis.setUserId(userId);
                singleChatHis.setTargetUserId(targetUserId);
                singleChatHis.setHisStatus(ChatStatusEnum.Ringing.getValue());
                singleChatHis.setHisCreateTime(curTime);
                singleChatHis.setHisPriceType(singleChatInfo.getSingleChatPriceType());
                singleChatHis.setHisPrice(singleChatInfo.getSingleChatPrice());
                singleChatHis.setChannelId(singleChatRequestDTO.getChannelId());
                singleChatHis.setHisIsMatch(BoolIntVal.False);
                singleChatHisDAO.insert(singleChatHis);

                // 创建群组
                ChatGroupDO chatGroupDO = new ChatGroupDO();
                chatGroupDO.setGroupId(hisId);
                chatGroupDO.setOwnerUserId(targetUserId);
                chatGroupDO.setGroupType(ChatGroupType.SINGLE_CHAT);
                chatGroupDO.setGroupCreateTime(curTime);
                chatGroupDAO.insert(chatGroupDO);

                // 给目标用户发送视频聊天消息
                SingleChatRequestMsg singleChatRequestMsg = new SingleChatRequestMsg();
                singleChatRequestMsg.setHisId(hisId);
                singleChatRequestMsg.setUserId(userId);
                imService.sendCustomImMsg(null, targetUserId, singleChatRequestMsg, null);

                // 呼叫超时消息
                QueueSingleChatRingTimeoutMsg timeoutMsg = new QueueSingleChatRingTimeoutMsg();
                timeoutMsg.setHisId(hisId);
                timeoutMsg.setIsMatch(false);
                rocketProducer.sendMsg(RocketTopic.TPC_DELAY, RocketTag.SINGLE_CHAT_RING_TIMEOUT, timeoutMsg.serialize(), curTime + TimeConstants.SECOND_30_MILLS);

                // 统计
                statPlatformService.addPlatformStat(singleChatRequestDTO.getChannelId(), StatPlatformTypeEnum.SINGLE_CHAT_TIMES_REQUEST, BigDecimal.ONE);

                return hisId;
            }
        } catch (Exception e) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            log.error("发送单聊请求失败", e);
        } finally {
            cacheService.delKey(lockKey);
        }

        return null;
    }

    /**
     * 主播发起单聊请求
     *
     * @param singleChatRequestDTO
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public String sendSingleChatRequestByAnchor(SingleChatRequestDTO singleChatRequestDTO) {
        Long userId = singleChatRequestDTO.getTargetUserId();
        Long anchorUserId = singleChatRequestDTO.getUserId();
        Long curTime = System.currentTimeMillis();

        AnchorSingleChatInfoVO singleChatInfo = anchorService.getSingleChatInfo(anchorUserId);
        if (singleChatInfo == null) {
            return null;
        }

        // 开启用户占用
        String lockKey = RedisChatKeys.getSingleChatLock(anchorUserId);
        try {
            if (cacheService.setIfAbsent(lockKey, String.valueOf(userId), TimeConstants.SECOND_30_MILLS, TimeUnit.MILLISECONDS)) {
                // 修改状态
                if (anchorDAO.updateSingleChatStatus(anchorUserId, SingleChatStatus.BUSY, SingleChatStatus.IDLE) == 0) {
                    // 解锁
                    cacheService.delKey(lockKey);
                    // 重新给用户匹配
                    chatMatchProvider.matchDelay(userId, TimeConstants.SECOND_15_MILLS);
                    return null;
                }

                // 创建记录
                String hisId = SecureUtil.md5(String.format("%d:%d:%d", userId, anchorUserId, curTime));
                SingleChatHisDO singleChatHis = new SingleChatHisDO();
                singleChatHis.setHisId(hisId);
                singleChatHis.setUserId(userId);
                singleChatHis.setTargetUserId(anchorUserId);
                singleChatHis.setHisStatus(ChatStatusEnum.Ringing.getValue());
                singleChatHis.setHisCreateTime(curTime);
                singleChatHis.setHisPriceType(singleChatInfo.getSingleChatPriceType());
                singleChatHis.setHisPrice(singleChatInfo.getSingleChatPrice());
                singleChatHis.setChannelId(singleChatRequestDTO.getChannelId());
                singleChatHis.setHisIsMatch(BoolIntVal.True);
                singleChatHisDAO.insert(singleChatHis);

                // 更新匹配记录
                singleChatMatchHisDAO.updateChatHisId(singleChatRequestDTO.getMatchId(), hisId);

                // 创建群组
                ChatGroupDO chatGroupDO = new ChatGroupDO();
                chatGroupDO.setGroupId(hisId);
                chatGroupDO.setOwnerUserId(anchorUserId);
                chatGroupDO.setGroupType(ChatGroupType.SINGLE_CHAT);
                chatGroupDO.setGroupCreateTime(curTime);
                chatGroupDAO.insert(chatGroupDO);

                // 给目标用户发送视频聊天消息
                SingleChatMatchMsg matchMsg = new SingleChatMatchMsg();
                matchMsg.setUserId(userId);
                matchMsg.setAnchorUserId(anchorUserId);
                matchMsg.setHisId(hisId);
                matchMsg.setShowVip(vipService.isVip(userId) ? BoolIntVal.False : BoolIntVal.True);
                matchMsg.setShowFree(hasGetFreeSingleChatMinutes(userId) ? BoolIntVal.False : BoolIntVal.True);
                imService.sendCustomImMsg(null, userId, matchMsg, null);

                // 呼叫超时消息
                QueueSingleChatRingTimeoutMsg timeoutMsg = new QueueSingleChatRingTimeoutMsg();
                timeoutMsg.setHisId(hisId);
                timeoutMsg.setIsMatch(true);
                rocketProducer.sendMsg(RocketTopic.TPC_DELAY, RocketTag.SINGLE_CHAT_RING_TIMEOUT, timeoutMsg.serialize(), curTime + TimeConstants.SECOND_30_MILLS);

                // 统计
                statPlatformService.addPlatformStat(singleChatRequestDTO.getChannelId(), StatPlatformTypeEnum.SINGLE_CHAT_TIMES_MATCH_REQUEST, BigDecimal.ONE);

                return hisId;
            }
        } catch (Exception e) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            log.error("发送单聊请求失败", e);
        } finally {
            cacheService.delKey(lockKey);
        }

        return null;
    }

    /**
     * 处理单聊请求
     *
     * @param hisId
     * @param decision
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public String decideSingleChatRequest(String hisId, int decision, String appVer) {
        SingleChatHisDO singleChatHis = singleChatHisDAO.selectById(hisId);
        if (singleChatHis == null) {
            return null;
        }

        boolean isMatch = Optional.ofNullable(singleChatHis.getHisIsMatch()).map(t -> t == BoolIntVal.True).orElse(false);

        if (BoolIntVal.True == decision) {
            // 接受
            int updated = singleChatHisDAO.updateStatus(hisId, ChatStatusEnum.Calling.getValue(), ChatStatusEnum.Ringing.getValue(), System.currentTimeMillis());
            if (updated == 0) {
                return null;
            }

            // 开始计费
            Long timestamp = System.currentTimeMillis();
            Long balance = payService.payForSingleChat(singleChatHis.getHisId(), 1, timestamp);
            if (balance == null || balance < 0) {
                return null;
            }


            // 获取 token
            String anchorToken = providerBuilder.provider(appVer).buildToken(hisId, singleChatHis.getTargetUserId(), AgoraProvider.Role.Role_Publisher.initValue, 0);

            // 获取另一个 token
            String userToken = providerBuilder.provider(appVer).buildToken(hisId, singleChatHis.getUserId(), AgoraProvider.Role.Role_Subscriber.initValue, 0);

            // 发送接受消息
            SingleChatDecisionMsg decisionMsg = new SingleChatDecisionMsg();
            decisionMsg.setDecision(decision);
            decisionMsg.setChannelName(hisId);
            decisionMsg.setToken(isMatch ? anchorToken : userToken);
            if (isMatch) {
                imService.sendCustomImMsg(null, singleChatHis.getTargetUserId(), decisionMsg, null);
            } else {
                imService.sendCustomImMsg(null, singleChatHis.getUserId(), decisionMsg, null);
            }

            // 周期计费延时消息
            QueueSingleChatChargingTickMsg singleChatChargingTickMsg = new QueueSingleChatChargingTickMsg();
            singleChatChargingTickMsg.setHisId(singleChatHis.getHisId());
            singleChatChargingTickMsg.setTimes(2);
            singleChatChargingTickMsg.setAppVer(appVer);
            rocketProducer.sendMsg(RocketTopic.TPC_DELAY, RocketTag.SINGLE_CHAT_CHARGING_TICK, singleChatChargingTickMsg.serialize(), timestamp + TimeConstants.MIN_MILLS);

            // 增加主播接通次数
            anchorDAO.increaseSingleChatConnectTimes(singleChatHis.getTargetUserId(), 1);

            // 统计
            if (isMatch) {
                statPlatformService.addPlatformStat(singleChatHis.getChannelId(), StatPlatformTypeEnum.SINGLE_CHAT_TIMES_MATCH_CONNECT, BigDecimal.ONE);
            } else {
                statPlatformService.addPlatformStat(singleChatHis.getChannelId(), StatPlatformTypeEnum.SINGLE_CHAT_TIMES_CONNECT, BigDecimal.ONE);
            }

            // 开始录制
            String res = providerBuilder.provider(appVer).startRecord(hisId);
            if (StringUtils.isNotBlank(res)) {
                JSONObject jsonObject = JSON.parseObject(res);
                singleChatHisDAO.updateReplayId(hisId, jsonObject.getString("sid"), jsonObject.getString("resourceId"));
            }

            return isMatch ? userToken : anchorToken;
        } else {
            // 恢复空闲
            anchorDAO.updateSingleChatStatus(singleChatHis.getTargetUserId(), SingleChatStatus.IDLE, SingleChatStatus.BUSY);

            // 拒绝
            singleChatHisDAO.updateStatus(hisId, ChatStatusEnum.Rejected.getValue(), ChatStatusEnum.Ringing.getValue(), System.currentTimeMillis());

            // 发送消息
            SingleChatDecisionMsg decisionMsg = new SingleChatDecisionMsg();
            decisionMsg.setDecision(decision);
            if (isMatch) {
                imService.sendCustomImMsg(null, singleChatHis.getTargetUserId(), decisionMsg, null);
            } else {
                imService.sendCustomImMsg(null, singleChatHis.getUserId(), decisionMsg, null);
            }

            // 将主播加回匹配列表
            chatMatchProvider.addMatchPoolAnchor(singleChatHis.getTargetUserId());

            // 解除锁定
            cacheService.delKey(RedisCacheKeys.getSingleChatMatchLock(singleChatHis.getUserId()));
            cacheService.delKey(RedisCacheKeys.getSingleChatMatchLock(singleChatHis.getTargetUserId()));

            return null;
        }
    }

    /**
     * 取消单聊请求
     *
     * @param hisId
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean cancelSingleChatRequest(String hisId) {
        try {
            SingleChatHisDO singleChatHis = singleChatHisDAO.selectById(hisId);
            if (singleChatHis == null) {
                return false;
            }

            // 更新记录
            int updated = singleChatHisDAO.updateStatus(hisId, ChatStatusEnum.Canceled.getValue(), ChatStatusEnum.Ringing.getValue(), System.currentTimeMillis());
            if (updated == 0) {
                return false;
            }

            // 更新用户状态
            anchorDAO.updateSingleChatStatus(singleChatHis.getTargetUserId(), SingleChatStatus.IDLE, SingleChatStatus.BUSY);


            // 发送取消消息
            SingleChatRingTimeoutMsg singleChatRingTimeoutMsg = new SingleChatRingTimeoutMsg();
            singleChatRingTimeoutMsg.setHisId(hisId);
            if (BoolIntVal.True == Optional.ofNullable(singleChatHis.getHisIsMatch()).orElse(BoolIntVal.False)) {
                imService.sendCustomImMsg(null, singleChatHis.getUserId(), singleChatRingTimeoutMsg, null);
            } else {
                imService.sendCustomImMsg(null, singleChatHis.getTargetUserId(), singleChatRingTimeoutMsg, null);
            }

            // 重新将主播加入池子
            chatMatchProvider.addMatchPoolAnchor(singleChatHis.getTargetUserId());

            // 解除锁定
            cacheService.delKey(RedisCacheKeys.getSingleChatMatchLock(singleChatHis.getUserId()));
            cacheService.delKey(RedisCacheKeys.getSingleChatMatchLock(singleChatHis.getTargetUserId()));

            return true;
        } catch (Exception e) {
            log.error("单聊请求取消失败", e);
        }
        return false;
    }

    /**
     * 停止单聊
     *
     * @param hisId
     * @param userId
     * @return
     */
    @Override
    public SingleChatHisDO stopSingleChat(String hisId, Long userId, String appVer) {
        SingleChatHisDO singleChatHis = singleChatHisDAO.selectById(hisId);
        SingleChatStopReasonEnum singleChatStopReason = SingleChatStopReasonEnum.AnchorClose;
        if (userId.equals(singleChatHis.getUserId())) {
            singleChatStopReason = SingleChatStopReasonEnum.UserClose;
        }
        return singleChatProvider.stopChat(hisId, singleChatStopReason, appVer);
    }

    /**
     * 获取单聊状态
     *
     * @param hisId
     * @param userId
     * @param targetUserId
     * @return
     */
    @Override
    public ChatStatusEnum getSingleChatStatus(String hisId, Long userId, Long targetUserId) {
        Integer hisStatus = singleChatHisDAO.selectStatus(hisId, userId, targetUserId);
        if (hisStatus != null) {
            return ChatStatusEnum.toEnum(hisStatus);
        }
        return null;
    }

    /**
     * 查询记录
     *
     * @param hisId
     * @return
     */
    @Override
    public SingleChatHisDO getSingleChatHis(String hisId) {
        return singleChatHisDAO.selectById(hisId);
    }

    /**
     * 判断是否有加入的权限
     *
     * @param hisId
     * @param userId
     * @return
     */
    @Override
    public boolean hasConnectPermission(String hisId, Long userId) {
        SingleChatHisDO singleChatHis = singleChatHisDAO.selectById(hisId);
        if (singleChatHis != null) {
            if (ChatStatusEnum.Calling != ChatStatusEnum.toEnum(singleChatHis.getHisStatus())) {
                // 非通话状态不可加入
                return false;
            }
            if (singleChatHis.getUserId().equals(userId)) {
                // 用户可以加入
                return true;
            }
            if (singleChatHis.getTargetUserId().equals(userId)) {
                // 主播可以加入
                return true;
            }
        }
        return false;
    }

    /***
     * 获取单聊推荐列表
     * @param num
     * @param userId
     * @param singleChatStatus
     * @param isOnline
     * @param channelId
     * @return
     */
    @Override
    public List<SingleChatUserVO> getSingleChatSuggestList(int num, Long userId, Integer singleChatStatus, Integer isOnline, Long channelId) {
        Long suggestChannelId = channelService.getSingleChatSuggestChannelId(channelId);

        // 渠道单聊分类
        String singleChatCategories = channelService.getSingleChatCategories(channelId);

        int count = singleChatSuggestDAO.count(singleChatStatus, isOnline, suggestChannelId, singleChatCategories);
        if (count > 0) {
            int pageNum = count / num;
            if (count % num > 0) {
                pageNum++;
            }

            int pageIndex = 1;
            if (pageNum > 1) {
                pageIndex = RandomUtil.randomInt(1, pageNum + 1);
            }
            PageList<SingleChatUserVO> pageList = new PageList<>(pageIndex, num);

            // 偏移量补足数量
            int offset = 0;
            if (pageIndex > 1 && count % num > 0 && pageIndex == pageNum) {
                offset = count % num - 1;
            }

            List<SingleChatUserVO> list = singleChatSuggestDAO.selectAll(pageList.getPageCursor() - offset, num, singleChatStatus, isOnline, suggestChannelId, singleChatCategories);
            if (CollectionUtil.isNotEmpty(list)) {
                for (SingleChatUserVO singleChatUserVO : list) {
                    if (StringUtils.isBlank(singleChatUserVO.getAreaCity())) {
                        singleChatUserVO.setAreaCity("北京市");
                    }
                    if (BoolIntVal.False == singleChatUserVO.getIsOnline() || LiveStatusEnum.Living == LiveStatusEnum.toEnum(singleChatUserVO.getLiveStatus())) {
                        singleChatUserVO.setSingleChatStatus(SingleChatStatus.BUSY);
                    }
                    singleChatUserVO.setSingleChatPriceTypeName(WalletTypeEnum.toEnum(singleChatUserVO.getSingleChatPriceType()).getName());
                }

                return list;
            }
        }

        return Collections.emptyList();
    }

    /**
     * 获取单聊列表
     *
     * @param pageIndex
     * @param pageSize
     * @param userSex
     * @param categoryId
     * @param singleChatStatus
     * @param isOnline
     * @param channelId
     * @return
     */
    @Override
    public PageList<SingleChatUserVO> getSingleChatList(int pageIndex, int pageSize, Integer userSex, Long categoryId, Integer singleChatStatus, Integer isOnline, Long channelId) {
        PageList<SingleChatUserVO> pageList = new PageList<>(pageIndex, pageSize);

        String singleChatCategories = channelService.getSingleChatCategories(channelId);

        List<SingleChatUserVO> list = anchorDAO.selectSingleChatUserList(pageList.getPageCursor(), pageSize, singleChatStatus, userSex, categoryId, null, singleChatCategories);
        if (CollectionUtil.isNotEmpty(list)) {
            for (SingleChatUserVO singleChatUserVO : list) {
                if (StringUtils.isBlank(singleChatUserVO.getAreaCity())) {
                    singleChatUserVO.setAreaCity("北京市");
                }
                if (BoolIntVal.False == singleChatUserVO.getIsOnline() || LiveStatusEnum.Living == LiveStatusEnum.toEnum(singleChatUserVO.getLiveStatus())) {
                    singleChatUserVO.setSingleChatStatus(SingleChatStatus.BUSY);
                }
                singleChatUserVO.setSingleChatPriceTypeName(WalletTypeEnum.toEnum(singleChatUserVO.getSingleChatPriceType()).getName());
            }
            pageList.setList(list);
        }

        pageList.setTotal(anchorDAO.countSingleChatUsers(singleChatStatus, userSex, categoryId, isOnline, singleChatCategories));
        return pageList;
    }

    /**
     * 获取附近单聊列表
     *
     * @param pageIndex
     * @param pageSize
     * @param longitude
     * @param latitude
     * @param distance
     * @param userSex
     * @param categoryId
     * @param singleChatStatus
     * @param channelId
     * @param userAgeMin
     * @param userAgeMax
     * @param areaId
     * @param isNew
     * @return
     */
    @Override
    public PageList<SingleChatUserVO> getSingleChatNearbyList(int pageIndex, int pageSize, Double longitude, Double latitude, Double distance, Integer userSex, Long categoryId, Integer singleChatStatus, Long channelId, Integer userAgeMin, Integer userAgeMax, Integer areaId, int isNew) {
        NearUserDTO nearUserDTO = new NearUserDTO();
        nearUserDTO.setPageIndex(pageIndex);
        nearUserDTO.setPageSize(pageSize);
        nearUserDTO.setUserSex(userSex);
        nearUserDTO.setSingleChatCategoryId(categoryId);
        nearUserDTO.setSingleChatEnable(BoolIntVal.True);
        nearUserDTO.setUserAge(userAgeMax);
        nearUserDTO.setUserLongitude(longitude);
        nearUserDTO.setUserLatitude(latitude);
        nearUserDTO.setDistance(distance);
        nearUserDTO.setAreaId(areaId);
        nearUserDTO.setIsNew(isNew);

        PageList<SingleChatUserVO> pageList = new PageList<>();

        PageList<NearbyUser> nearbyUserPageList = userSearchService.getNearUserPageList(nearUserDTO);
        if (nearbyUserPageList.getTotal() > 0) {
            pageList.setTotal(nearbyUserPageList.getTotal());
            List<Long> userIds = new ArrayList<>();
            Map<Long, Double> distanceMap = new HashMap<>();

            for (NearbyUser nearbyUser : nearbyUserPageList.getList()) {
                userIds.add(nearbyUser.getUserId());
                distanceMap.put(nearbyUser.getUserId(), nearbyUser.getDistance());
            }

            if (CollectionUtil.isNotEmpty(userIds)) {
                // 查询记录
                List<SingleChatUserVO> singleChatUserList = anchorDAO.selectSingleChatUserListByIds(userIds);
                for (SingleChatUserVO singleChatUser : singleChatUserList) {
                    if (BoolIntVal.False == singleChatUser.getIsOnline() || LiveStatusEnum.Living == LiveStatusEnum.toEnum(singleChatUser.getLiveStatus())) {
                        singleChatUser.setSingleChatStatus(SingleChatStatus.BUSY);
                    }
                    singleChatUser.setDistance(distanceMap.get(singleChatUser.getUserId()));
                    singleChatUser.setSingleChatPriceTypeName(WalletTypeEnum.toEnum(singleChatUser.getSingleChatPriceType()).getName());
                    singleChatUser.setUserConstellation(userService.getUserConstellation(singleChatUser.getUserId()));
                    singleChatUser.setLastOnlineTime(userService.getLastOnlineTime(singleChatUser.getUserId()));
                }

                pageList.setList(singleChatUserList);
            }
        }

        return pageList;
    }

    /**
     * 随机获取单聊用户
     *
     * @param num
     * @param userId
     * @param userSex
     * @param categoryId
     * @param singleChatStatus
     * @param channelId
     * @return
     */
    @Override
    public PageList<SingleChatUserVO> getSingleChatRandomList(int num, Long userId, Integer userSex, Long categoryId, Integer singleChatStatus, Integer isOnline, Long channelId) {

        String singleChatCategories = channelService.getSingleChatCategories(channelId);

        int count = anchorDAO.countSingleChatUsers(singleChatStatus, userSex, categoryId, isOnline, singleChatCategories);
        if (count > 0) {
            int pageNum = count / num;
            if (count % num > 0) {
                pageNum++;
            }

            int pageIndex = 1;
            if (pageNum > 1) {
                pageIndex = RandomUtil.randomInt(1, pageNum + 1);
            }
            PageList<SingleChatUserVO> pageList = new PageList<>(pageIndex, num);
            // 偏移量补足数量
            int offset = 0;
            if (pageIndex > 1 && count % num > 0 && pageIndex == pageNum) {
                offset = count % num - 1;
            }
            List<SingleChatUserVO> list = anchorDAO.selectSingleChatUserList(pageList.getPageCursor() - offset, num, singleChatStatus, userSex, categoryId, isOnline, singleChatCategories);
            if (CollectionUtil.isNotEmpty(list)) {
                for (SingleChatUserVO singleChatUserVO : list) {
                    if (StringUtils.isBlank(singleChatUserVO.getAreaCity())) {
                        singleChatUserVO.setAreaCity("北京市");
                    }
                    singleChatUserVO.setSingleChatPriceTypeName(WalletTypeEnum.toEnum(singleChatUserVO.getSingleChatPriceType()).getName());
                }

                pageList.setList(list);
                pageList.setTotal(count);
                return pageList;
            }
        }
        return new PageList<>();
    }

    /**
     * 判断是否已评价
     *
     * @param hisId
     * @param userId
     * @return
     */
    @Override
    public boolean isSingleChatEvaluateExists(String hisId, Long userId) {
        return singleChatEvaluateDAO.countById(hisId, userId) > 0;
    }

    /**
     * 创建单聊评价
     *
     * @param hisId
     * @param userId
     * @param targetUserId
     * @param scoreValue
     * @param tagIds
     * @param isAnchor
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean addSingleChatEvaluate(String hisId, Long userId, Long targetUserId, Integer scoreValue, String tagIds, boolean isAnchor) {
        try {
            Long curTime = System.currentTimeMillis();

            // 创建评价
            SingleChatEvaluateDO singleChatEvaluate = new SingleChatEvaluateDO();
            singleChatEvaluate.setHisId(hisId);
            singleChatEvaluate.setUserId(userId);
            singleChatEvaluate.setTargetUserId(targetUserId);
            singleChatEvaluate.setScoreValue(scoreValue);
            singleChatEvaluate.setTagIds(tagIds);
            singleChatEvaluate.setEvaluateTime(curTime);
            singleChatEvaluateDAO.insert(singleChatEvaluate);

            // 增加用户标签统计
            if (StringUtils.isNotBlank(tagIds)) {
                List<Long> tagIdList = Arrays.stream(tagIds.split(",")).map(t -> Convert.toLong(t)).collect(Collectors.toList());
                List<UserSingleChatTagDO> userSingleChatTagList = new ArrayList<>();
                for (Long tagId : tagIdList) {
                    UserSingleChatTagDO userSingleChatTag = new UserSingleChatTagDO();
                    userSingleChatTag.setUserId(targetUserId);
                    userSingleChatTag.setTagId(tagId);
                    userSingleChatTag.setTagCount(1);
                    userSingleChatTag.setUpdateTime(curTime);

                    userSingleChatTagList.add(userSingleChatTag);
                }

                userSingleChatTagDAO.batchInsertOrUpdate(userSingleChatTagList);
            }

            if (!isAnchor) {
                // 更新评分
                anchorDAO.increaseSingleChatScore(targetUserId, scoreValue);

                // 增加评价次数
                anchorDAO.increaseSingleChatEvaluateTimes(targetUserId, 1);
            }

            return true;
        } catch (Exception e) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            log.error("单聊评价创建失败", e);
        }
        return false;
    }

    /**
     * 获取单聊主播
     *
     * @param userId
     * @return
     */
    @Override
    public SingleChatUserVO getSingleChatUser(Long userId) {
        SingleChatUserVO singleChatUserVO = anchorDAO.selectSingleChatUserById(userId);
        if (singleChatUserVO != null) {
            singleChatUserVO.setSingleChatPriceTypeName(WalletTypeEnum.toEnum(singleChatUserVO.getSingleChatPriceType()).getName());
            return singleChatUserVO;
        }
        return null;
    }

    /**
     * 获取单聊匹配记录
     *
     * @param num
     * @return
     */
    @Override
    public List<SingleChatMatchedHis> getSingleChatMatchedHisList(int num) {
        List<SingleChatMatchedHis> list = singleChatHisDAO.selectMatchedHis(num, Arrays.asList(ChatStatusEnum.Calling.getValue(), ChatStatusEnum.Closed.getValue()));
        return list;
    }

    /**
     * 刷新提醒
     *
     * @param hisId
     * @return
     */
    @Override
    public Long refreshSingleChatRemind(String hisId) {
        SingleChatHisDO singleChatHis = singleChatHisDAO.selectById(hisId);
        if (singleChatHis != null) {
            // 计算用户余额
            long minutesRemain = walletService.getBalance(singleChatHis.getUserId(), singleChatHis.getHisPriceType()) / singleChatHis.getHisPrice();
            minutesRemain += walletService.getChatTimeWalletMinutes(singleChatHis.getUserId(), ChatTimeWalletType.SINGLE_CHAT);
            // 发送余额提醒
            SingleChatBalanceRemindMsg singleChatBalanceRemindMsg = new SingleChatBalanceRemindMsg();
            singleChatBalanceRemindMsg.setHisId(singleChatHis.getHisId());
            singleChatBalanceRemindMsg.setMinutesRemain(minutesRemain);
            imService.sendChatGroupMsg(ImTargetType.GROUP, null, null, singleChatHis.getHisId(), singleChatBalanceRemindMsg);
            return minutesRemain;
        }
        return null;
    }

    /**
     * 验签
     *
     * @param signature
     * @return
     */
    @Override
    public boolean verifyAgoraNotifySign(RtcProviderSignature signature, String appVer) {
        return providerBuilder.provider(appVer).verifyNotifySign(signature);
    }

    /**
     * 处理三方RTC的网络回调
     *
     * @param callback
     * @return
     */
    @Override
    public boolean handleCallbackNotify(RtcServiceCallback callback) {
        switch (callback.getEventType().intValue()) {
            case AgoraRealTimeEventType.ANCHOR_JOIN_CHANNEL:
            case AgoraRealTimeEventType.USER_JOIN_COMMUNICATION_MODE:
            case ZegoServerCallbackType.StreamCreate:
                QueueChatUserJoinMsg chatUserJoinMsg = new QueueChatUserJoinMsg();
                chatUserJoinMsg.setChannelName(callback.getChannelName());
                chatUserJoinMsg.setJoinUserId(callback.getUserID());
                chatUserJoinMsg.setPlatform(callback.getPlatform());
                chatUserJoinMsg.setTimestamp(callback.getTimestamp());
                rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.CHAT_USER_JOIN, chatUserJoinMsg.serialize());
                break;
            case AgoraRealTimeEventType.ANCHOR_LEAVE_CHANNEL:
            case AgoraRealTimeEventType.USER_LEAVE_COMMUNICATION_MODE:
            case ZegoServerCallbackType.StreamClose:
                QueueChatUserExitMsg chatUserExitMsg = new QueueChatUserExitMsg();
                chatUserExitMsg.setChannelName(callback.getChannelName());
                chatUserExitMsg.setExitUserId(callback.getUserID());
                chatUserExitMsg.setReason(callback.getReason());
                chatUserExitMsg.setTimestamp(callback.getTimestamp());
                chatUserExitMsg.setAppVer(callback.getAppVer());
                rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.CHAT_USER_EXIT, chatUserExitMsg.serialize());
                break;
            default:
                return true;
        }

        return true;
    }

    /**
     * 获取 token
     *
     * @param channelName
     * @param userId
     * @param role
     * @param expireTime
     * @return
     */
    @Override
    public String getAgoraToken(String channelName, Long userId, Integer role, long expireTime, String appVer) {
        return providerBuilder.provider(appVer).buildToken(channelName, userId, role, expireTime);
    }

    /**
     * 查询匹配记录
     *
     * @param matchId
     * @return
     */
    @Override
    public SingleChatMatchHisDO getSingleChatMatchHis(String matchId) {
        return singleChatMatchHisDAO.selectById(matchId);
    }

    /**
     * 领取免费时长
     *
     * @param freeSingleChatMinutesDTO
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean getFreeSingleChatMinutes(FreeSingleChatMinutesDTO freeSingleChatMinutesDTO) {
        try {
            Long userId = freeSingleChatMinutesDTO.getUserId();
            SingleChatFreeHisDO singleChatFreeHis = new SingleChatFreeHisDO();
            singleChatFreeHis.setUserId(userId);
            singleChatFreeHis.setMinutes(freeSingleChatMinutesDTO.getMinutes());
            singleChatFreeHis.setChannelId(freeSingleChatMinutesDTO.getChannelId());
            singleChatFreeHis.setMatchId(freeSingleChatMinutesDTO.getMatchId());
            singleChatFreeHis.setChatHisId(freeSingleChatMinutesDTO.getChatHisId());
            singleChatFreeHis.setHisCreateTime(System.currentTimeMillis());
            singleChatFreeHisDAO.insert(singleChatFreeHis);

            walletService.addChatTime(userId, ChatTimeWalletType.SINGLE_CHAT, System.currentTimeMillis() + TimeConstants.DAY_MILLS, freeSingleChatMinutesDTO.getMinutes(), "新用户免费领取", freeSingleChatMinutesDTO.getChatHisId());

            return true;
        } catch (Exception e) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            log.error("免费时长领取失败", e);
        }
        return false;
    }

    /**
     * 判断是否领取过免费时长
     *
     * @param userId
     * @return
     */
    @Override
    public boolean hasGetFreeSingleChatMinutes(Long userId) {
        return singleChatFreeHisDAO.countById(userId) > 0;
    }

    @Override
    public PageList<SingleChatUserVO> getVipSingleChatList(VipSearchDTO searchDTO) {
        if (searchDTO.getSingleChatStatus() != null && SingleChatStatus.IDLE == searchDTO.getSingleChatStatus()) {
            searchDTO.setIsOnline(BoolIntVal.True);
            searchDTO.setLiveStatus(LiveStatusEnum.Closed.getValue());
        }

        PageList<SingleChatUserVO> page = new PageList<>(searchDTO.getPageIndex(), searchDTO.getPageSize());
        searchDTO.setPageCursor(page.getPageCursor());

        List<SingleChatUserVO> list = anchorDAO.selectVipSingleChatList(searchDTO);

        if (CollectionUtils.isEmpty(list)) {
            page.setList(new ArrayList<>());
            page.setTotal(0);
            return page;
        }

        if (CollectionUtil.isNotEmpty(list)) {
            for (SingleChatUserVO singleChatUserVO : list) {
                if (StringUtils.isBlank(singleChatUserVO.getAreaCity())) {
                    singleChatUserVO.setAreaCity("北京市");
                }
                if (BoolIntVal.False == singleChatUserVO.getIsOnline() || LiveStatusEnum.Living == LiveStatusEnum.toEnum(singleChatUserVO.getLiveStatus())) {
                    singleChatUserVO.setSingleChatStatus(SingleChatStatus.BUSY);
                }
                singleChatUserVO.setSingleChatPriceTypeName(WalletTypeEnum.toEnum(singleChatUserVO.getSingleChatPriceType()).getName());
            }
        }
        page.setList(list);
        page.setTotal(anchorDAO.countVipSingleChatList(searchDTO));

        return page;
    }

}
