package com.starbuds.server.service.media.provider;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.db.GameGroupDO;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.service.media.dao.GameGroupDAO;
import com.starbuds.server.service.media.pojo.qo.GameGroupPageListQO;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;

import static com.starbuds.server.common.pojo.util.PageListUtil.buildPageListQueryWrapper;

/**
 * 游戏分组
 */
@Component
@Slf4j
public class GameGroupProvider {

    /**
     * 主键
     */
    private String PRIMARY_KEY = "gameGroupId";

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Resource
    private GameGroupDAO gameGroupDAO;

    /**
     * 插入数据
     *
     * @param po
     * @return null:插入失败
     */
    @Transactional(rollbackFor = Exception.class)
    public GameGroupDO create(GameGroupDO po) {
        po.setGameGroupId(identityService.getId());
        po.setCreateTime(System.currentTimeMillis());
        po.setUpdateTime(po.getCreateTime());
        boolean flag = gameGroupDAO.insert(po) == 1;

        String json = JSON.toJSONString(po);
        if (flag) {
            log.info("创建成功 {}", json);
            return po;
        }
        log.warn("创建失败 {}", json);
        return null;
    }

    /**
     * 插入批量数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createBatch(List<GameGroupDO> list) {
        boolean flag = gameGroupDAO.insertBatch(list) > 0;
        return flag;
    }

    /**
     * 插入或更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createOrUpdate(GameGroupDO po) {
        boolean flag = gameGroupDAO.insertOrUpdate(po) == 1;
        return flag;
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean updateById(GameGroupDO po) {
        po.setCreateTime(null);
        po.setUpdateTime(System.currentTimeMillis());

        UpdateWrapper uw = new UpdateWrapper();
        uw.setPo(po);
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, po.getGameGroupId());
        uw.setQw(qw);
        return update(uw);
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean update(UpdateWrapper uw) {
        boolean flag = gameGroupDAO.update(uw) > 0;
        return flag;
    }

    /**
     * 根据id查询
     */
    public GameGroupDO getById(Long id) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return gameGroupDAO.selectOne(qw);
    }

    /**
     * 判断是否存在
     *
     * @param id 主键
     * @return true:存在
     */
    public boolean exist(Long id) {
        if (id == null) {
            return false;
        }
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return gameGroupDAO.selectCount(qw) > 0;
    }

    /**
     * 分页查询
     */
    public PageList<GameGroupDO> getPageList(GameGroupPageListQO qo) {
        QueryWrapper qw = buildPageListQueryWrapper(qo);
        PageList<GameGroupDO> page = new PageList<>(qo.getPageIndex(), qo.getPageSize());
        qw.setPageCursor(page.getPageCursor());
        qw.setPageSize(page.getPageSize());
        page.setList(gameGroupDAO.selectPageList(qw));
        page.setTotal(gameGroupDAO.selectCount(qw));
        return page;
    }

    // 手动编码开始

    /**
     * 获得所有游戏分组
     *
     * @return
     */
    public List<GameGroupDO> getAllList() {
        return gameGroupDAO.selectList(new QueryWrapper());
    }
}