package com.starbuds.server.service.media.provider;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.pojo.db.GameInfoDO;
import com.starbuds.server.service.media.dao.GameInfoDAO;

import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.service.media.pojo.qo.GameInfoPageListQO;
import lombok.extern.slf4j.Slf4j;

import javax.annotation.Resource;
import java.util.List;

import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.service.core.IdentityService;

import static com.starbuds.server.common.pojo.util.PageListUtil.buildPageListQueryWrapper;

import org.springframework.stereotype.Component;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.transaction.annotation.Transactional;

/**
 * 游戏信息
 */
@Component
@Slf4j
public class GameInfoProvider {

    /**
     * 主键
     */
    private String PRIMARY_KEY = "gameInfoId";

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Resource
    private GameInfoDAO gameInfoDAO;

    /**
     * 插入数据
     *
     * @param po
     * @return null:插入失败
     */
    @Transactional(rollbackFor = Exception.class)
    public GameInfoDO create(GameInfoDO po) {
        po.setGameInfoId(identityService.getId());
        po.setCreateTime(System.currentTimeMillis());
        po.setUpdateTime(po.getCreateTime());
        boolean flag = gameInfoDAO.insert(po) == 1;

        String json = JSON.toJSONString(po);
        if (flag) {
            log.info("创建成功 {}", json);
            return po;
        }
        log.warn("创建失败 {}", json);
        return null;
    }

    /**
     * 插入批量数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createBatch(List<GameInfoDO> list) {
        boolean flag = gameInfoDAO.insertBatch(list) > 0;
        return flag;
    }

    /**
     * 插入或更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createOrUpdate(GameInfoDO po) {
        boolean flag = gameInfoDAO.insertOrUpdate(po) == 1;
        return flag;
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean updateById(GameInfoDO po) {
        po.setCreateTime(null);
        po.setUpdateTime(System.currentTimeMillis());

        UpdateWrapper uw = new UpdateWrapper();
        uw.setPo(po);
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, po.getGameInfoId());
        uw.setQw(qw);
        return update(uw);
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean update(UpdateWrapper uw) {
        boolean flag = gameInfoDAO.update(uw) > 0;
        return flag;
    }

    /**
     * 根据id查询
     */
    public GameInfoDO getById(Long id) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return gameInfoDAO.selectOne(qw);
    }

    /**
     * 判断是否存在
     *
     * @param id 主键
     * @return true:存在
     */
    public boolean exist(Long id) {
        if (id == null) {
            return false;
        }
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return gameInfoDAO.selectCount(qw) > 0;
    }

    /**
     * 分页查询
     */
    public PageList<GameInfoDO> getPageList(GameInfoPageListQO qo) {
        QueryWrapper qw = buildPageListQueryWrapper(qo);
        PageList<GameInfoDO> page = new PageList<>(qo.getPageIndex(), qo.getPageSize());
        qw.setPageCursor(page.getPageCursor());
        qw.setPageSize(page.getPageSize());
        page.setList(gameInfoDAO.selectPageList(qw));
        page.setTotal(gameInfoDAO.selectCount(qw));
        return page;
    }

    // 手动编码开始

    /**
     * 获得游戏列表
     *
     * @param qw
     * @return
     */
    public List<GameInfoDO> getList(QueryWrapper qw) {
        return gameInfoDAO.selectList(qw);
    }
}