package com.starbuds.server.service.media.provider;

import cn.hutool.core.convert.Convert;
import com.starbuds.server.common.constant.cache.RedisLiveKeys;
import com.starbuds.server.common.service.core.CacheService;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;

import java.util.Optional;

/**
 * 备注
 *
 * @author Lay
 * @date 2020/1/2
 */
@Component
public class LivePopularityProvider {

    @Reference(version = "1.0.0")
    CacheService cacheService;

    public void init(Long userId) {
        remove(userId);
    }

    public void remove(Long userId) {
        // 移除
        cacheService.zRemove(RedisLiveKeys.LIVE_POPULARITY_SET, String.valueOf(userId));
    }

    public Long increase(Long userId, Long value) {
        return change(userId, Math.abs(value));
    }

    public Long decrease(Long userId, Long value) {
        return change(userId, -Math.abs(value));
    }

    private Long change(Long userId, Long value) {
        if (value < 0 && Optional.ofNullable(cacheService.zScore(RedisLiveKeys.LIVE_POPULARITY_SET, String.valueOf(userId))).orElse(0D) < 1) {
            return 0L;
        }
        Long newPopularity = Convert.toLong(cacheService.zIncrementScore(RedisLiveKeys.LIVE_POPULARITY_SET, String.valueOf(userId), Convert.toDouble(value)));
        if (newPopularity > Optional.ofNullable(cacheService.hashGet(RedisLiveKeys.LIVE_POPULARITY_MAX, String.valueOf(userId))).map(t -> Convert.toLong(t)).orElse(0L)) {
            cacheService.hashPut(RedisLiveKeys.LIVE_POPULARITY_MAX, String.valueOf(userId), String.valueOf(newPopularity));
        }
        return newPopularity;
    }

    public Long get(Long userId) {
        return Optional.ofNullable(cacheService.zScore(RedisLiveKeys.LIVE_POPULARITY_SET, String.valueOf(userId))).map(t -> Convert.toLong(t)).orElse(0L);
    }

    public Long getMax(Long userId) {
        return Optional.ofNullable(cacheService.hashGet(RedisLiveKeys.LIVE_POPULARITY_MAX, String.valueOf(userId))).map(t -> Convert.toLong(t)).orElse(0L);
    }
}
