package com.starbuds.server.service.media.rocket;

import com.aliyun.openservices.ons.api.Consumer;
import com.aliyun.openservices.ons.api.ONSFactory;
import com.aliyun.openservices.ons.api.PropertyKeyConst;
import com.aliyun.openservices.ons.api.PropertyValueConst;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.service.media.rocket.listener.DelayMsgListener;
import com.starbuds.server.service.media.rocket.listener.NormalMsgListener;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.DisposableBean;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.List;
import java.util.Properties;

/**
 * Class
 *
 * @author Lay
 * @date 2019/05/30
 */
@Component
@Slf4j
public class RocketConsumer implements DisposableBean {
    @Resource
    RocketConfig rocketConfig;

    @Resource
    DelayMsgListener delayMsgListener;

    @Resource
    NormalMsgListener normalMsgListener;

    private Consumer consumer;

    @Override
    public void destroy() throws Exception {
        if (consumer != null) {
            consumer.shutdown();
        }
    }

    public void start() {
        log.info("RocketMQ 消费者初始化");
        Properties properties = rocketConfig.getProps();

        // 集群模式订阅，共同分担消息处理
        properties.put(PropertyKeyConst.MessageModel, PropertyValueConst.CLUSTERING);
        consumer = ONSFactory.createConsumer(properties);

        log.info("RocketMQ 消费者启动");
        consumer.subscribe(RocketTopic.TPC_DELAY, getSubscribeTags(Arrays.asList(
                RocketTag.LIVE_HEAT_EXPIRE,
                RocketTag.LIVE_TIMEOUT,
                RocketTag.SINGLE_CHAT_RING_TIMEOUT,
                RocketTag.SINGLE_CHAT_CHARGING_TICK,
                RocketTag.SINGLE_CHAT_MATCH_DELAY,
                RocketTag.BATTLE_INVITE_TIMEOUT,
                RocketTag.BATTLE_WAIT_TIMEOUT,
                RocketTag.BATTLE_STAGE_DELAY,
                RocketTag.VOICE_MATCH_TIMEOUT,
                RocketTag.VOICE_MATCH_WAIT_TIMEOUT,
                RocketTag.VOICE_CHAT_CHARGING_TICK
        )), delayMsgListener);
        consumer.subscribe(RocketTopic.TPC_NORMAL, getSubscribeTags(Arrays.asList(
                RocketTag.GIFT_PAID,
                RocketTag.GIFT_COMBO_TIMEOUT,
                RocketTag.LIVE_USER_JOIN,
                RocketTag.LIVE_USER_EXIT,
                RocketTag.LIVE_START,
                RocketTag.LIVE_STOP,
                RocketTag.STREAM_PUSH,
                RocketTag.STREAM_CLOSE,
                RocketTag.SINGLE_CHAT_USER_JOIN,
                RocketTag.SINGLE_CHAT_USER_EXIT,
                RocketTag.CHAT_USER_JOIN,
                RocketTag.CHAT_USER_EXIT,
                RocketTag.USER_ONLINE_CHANGE,
                RocketTag.VOICE_MATCH_REQUEST,
                RocketTag.VOICE_CHAT_REJECT,
                RocketTag.VOICE_CHAT_START,
                RocketTag.VOICE_CHAT_STOP
        )), normalMsgListener);
        consumer.start();
    }

    private String getSubscribeTags(List<String> tags) {
        int size = tags.size();
        StringBuilder sbTags = new StringBuilder();
        for (int i = 0; i < size; i++) {
            sbTags.append(tags.get(i));
            if (i != size - 1) {
                sbTags.append("||");
            }
        }
        return sbTags.toString();
    }
}
