package com.starbuds.server.service.media.rocket.handler;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.crypto.SecureUtil;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.cache.RedisCacheKeys;
import com.starbuds.server.common.constant.cache.RedisChatKeys;
import com.starbuds.server.common.constant.cache.RedisFinanceKeys;
import com.starbuds.server.common.constant.cache.RedisLiveKeys;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.common.TimeConstants;
import com.starbuds.server.common.constant.data.MatchModeEnum;
import com.starbuds.server.common.constant.data.TaskStatusEnum;
import com.starbuds.server.common.constant.data.TaskTypeEnum;
import com.starbuds.server.common.constant.finance.ChatTimeWalletType;
import com.starbuds.server.common.constant.finance.WalletTypeEnum;
import com.starbuds.server.common.constant.media.*;
import com.starbuds.server.common.constant.message.im.ImTargetType;
import com.starbuds.server.common.constant.message.push.ModuleTypeEnum;
import com.starbuds.server.common.constant.message.push.PushActionType;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.constant.statistics.StatPlatformTypeEnum;
import com.starbuds.server.common.constant.user.VoiceChatStatus;
import com.starbuds.server.common.pojo.db.*;
import com.starbuds.server.common.pojo.message.im.chat.*;
import com.starbuds.server.common.pojo.message.im.live.*;
import com.starbuds.server.common.pojo.message.push.PushMsgParam;
import com.starbuds.server.common.pojo.mq.*;
import com.starbuds.server.common.pojo.user.AnchorInfoVO;
import com.starbuds.server.common.pojo.user.UserCardVO;
import com.starbuds.server.common.service.activity.LotteryActivityService;
import com.starbuds.server.common.service.core.CacheService;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.data.MountsService;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.data.UserSearchService;
import com.starbuds.server.common.service.finance.VipService;
import com.starbuds.server.common.service.finance.WalletService;
import com.starbuds.server.common.service.media.GiftService;
import com.starbuds.server.common.service.media.VoiceChatService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.statistics.StatPlatformService;
import com.starbuds.server.common.service.task.PushService;
import com.starbuds.server.common.service.user.AnchorService;
import com.starbuds.server.common.service.user.FamilyService;
import com.starbuds.server.common.service.user.SocialRelationService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.service.media.dao.*;
import com.starbuds.server.service.media.provider.*;
import com.starbuds.server.service.media.rocket.RocketProducer;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

/**
 * 普通队列消息处理
 *
 * @author Lay
 * @date 2020/1/2
 */
@Component
@Slf4j
public class NormalMsgHandler {

    @Reference(version = "1.0.0")
    CacheService cacheService;

    @Reference(version = "1.0.0")
    ImService imService;

    @Reference(version = "1.0.0")
    GiftService giftService;

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    AnchorService anchorService;

    @Reference(version = "1.0.0")
    UserSearchService userSearchService;

    @Reference(version = "1.0.0")
    MountsService mountsService;

    @Reference(version = "1.0.0")
    StatPlatformService statPlatformService;

    @Reference(version = "1.0.0")
    WalletService walletService;

    @Reference(version = "1.0.0")
    VipService vipService;

    @Reference(version = "1.0.0")
    LotteryActivityService lotteryActivityService;

    @Reference(version = "1.0.0")
    SocialRelationService socialRelationService;

    @Reference(version = "1.0.0")
    PushService pushService;

    @Reference(version = "1.0.0")
    IdentityService identityService;

    @Reference(version = "1.0.0")
    RedisService redisService;

    @Reference(version = "1.0.0")
    VoiceChatService voiceChatService;

    @Reference(version = "1.0.0")
    FamilyService familyService;

    @Resource
    LiveHeatProvider liveHeatProvider;

    @Resource
    LivePopularityProvider livePopularityProvider;

    @Resource
    RankProvider rankProvider;

    @Resource
    QiniuLiveProvider qiniuLiveProvider;

    @Resource
    SingleChatProvider singleChatProvider;

    @Resource
    ChatMatchProvider chatMatchProvider;

    @Resource
    BattleProvider battleProvider;

    @Resource
    LiveStayHisDAO liveStayHisDAO;

    @Resource
    LiveUserJoinHisDAO liveUserJoinHisDAO;

    @Resource
    SingleChatHisDAO singleChatHisDAO;

    @Resource
    LiveStreamHisDAO liveStreamHisDAO;

    @Resource
    VoiceChatProvider voiceChatProvider;

    @Resource
    VoiceChatHisDAO voiceChatHisDAO;

    @Resource
    ProviderBuilder providerBuilder;

    @Resource
    RocketProducer rocketProducer;

    /**
     * 连击持续时间
     */
    private static int COMBO_TIMEOUT = 5;

    /**
     * 礼物支付
     *
     * @param msg
     */
    public void handleGiftPaid(QueueGiftPaidMsg msg) {
        GiftSceneTypeEnum giftSceneType = GiftSceneTypeEnum.toEnum(msg.getGiftSceneType());

        // 处理 combo
        String comboKey = RedisCacheKeys.getGiftComboKey(msg.getReceiverUserId(), msg.getSenderUserId(), msg.getGiftId(), msg.getQuantity());

        long combo = cacheService.increaseAndExpire(comboKey, 1, COMBO_TIMEOUT, TimeUnit.SECONDS);
        if (combo == 1) {
            // 创建一个新的缓存
            cacheService.increaseAndExpire(RedisCacheKeys.getLiveComboHis(comboKey, System.currentTimeMillis()), 1, COMBO_TIMEOUT * 2, TimeUnit.SECONDS);

            // 缓存场景
            if (GiftSceneTypeEnum.SingleChat == giftSceneType) {
                cacheService.hashPut(RedisCacheKeys.GIFT_COMBO_CHAT_GROUP, comboKey, msg.getChannelName());
            }
        } else {
            // 获取最新的缓存 key
            List<String> keys = cacheService.getKeys(RedisCacheKeys.getLiveComboHisPattern(comboKey));
            if (CollectionUtil.isNotEmpty(keys)) {
                if (keys.size() == 1) {
                    cacheService.increaseAndExpire(keys.get(0), 1, COMBO_TIMEOUT * 2, TimeUnit.SECONDS);
                } else {
                    // 按时间排序并找出最近的那个
                    long timestamp = 0L;
                    for (String key : keys) {
                        long time = Convert.toLong(key.substring(key.lastIndexOf("_") + 1));
                        if (time > timestamp) {
                            timestamp = time;
                        }
                    }
                    cacheService.increaseAndExpire(RedisCacheKeys.getLiveComboHis(comboKey, timestamp), 1, COMBO_TIMEOUT * 2, TimeUnit.SECONDS);
                }
            }
        }

        // 查询礼物消息
        GiftDO giftDO = giftService.getGift(msg.getGiftId(), null);

        // 弹幕礼物策略
        LiveGiftCommentMsg liveGiftCommentMsg = null;
        if (BoolIntVal.True == Optional.ofNullable(giftDO.getIsComment()).orElse(BoolIntVal.False)) {
            boolean comboFirst = combo == 1 && BoolIntVal.True == Optional.ofNullable(giftDO.getCommentFirstCombo()).orElse(BoolIntVal.False);
            Integer commentComboMultiple = Optional.ofNullable(giftDO.getCommentComboMultiple()).orElse(0);
            boolean comboMultiple = commentComboMultiple > 0 && combo % commentComboMultiple == 0;
            if (comboFirst || comboMultiple) {
                liveGiftCommentMsg = new LiveGiftCommentMsg();
                liveGiftCommentMsg.setGiftId(msg.getGiftId());
                liveGiftCommentMsg.setGiftName(giftDO.getGiftName());
                liveGiftCommentMsg.setQuantity(msg.getQuantity());
                liveGiftCommentMsg.setCombo(combo);
            }
        }

        // 直播场景
        if (GiftSceneTypeEnum.Live == giftSceneType) {
            // 星钻礼物才加榜，活动礼物不加榜
            boolean isActivityGift = giftDO.getGiftType() == GiftTypeEnum.LOTTERY.getValue();
            if (!isActivityGift && anchorService.isAnchor(msg.getReceiverUserId()) && WalletTypeEnum.STAR_DIAMOND.getValue() == msg.getGiftPriceType()) {
                rankProvider.addLiveRankAmount(msg.getReceiverUserId(), msg.getSenderUserId(), msg.getAnchorAmount());
            }

            // 发送礼物消息到房间
            LiveGiftComboMsg liveGiftComboMsg = new LiveGiftComboMsg();
            liveGiftComboMsg.setGiftId(msg.getGiftId());
            liveGiftComboMsg.setGiftName(giftDO.getGiftName());
            liveGiftComboMsg.setQuantity(msg.getQuantity());
            liveGiftComboMsg.setCombo(combo);
            liveGiftComboMsg.setComboKeyHash(SecureUtil.md5(comboKey));
            liveGiftComboMsg.setWinAmount(msg.getWinAmount());
            liveGiftComboMsg.setWinMultipleMax(msg.getWinMultipleMax());
            liveGiftComboMsg.setSourceType(msg.getSourceType());
            imService.sendLiveRoomMsg(ImTargetType.GROUP, msg.getSenderUserId(), msg.getReceiverUserId(), liveGiftComboMsg);

            // 礼物热度
            if (giftDO.getHeatValue() != null && giftDO.getHeatValue() > 0) {
                liveHeatProvider.changeLiveHeat(msg.getReceiverUserId(), giftDO.getHeatValue(), giftDO.getHeatDuration());
            }

            // 礼物弹幕
            if (liveGiftCommentMsg != null) {
                imService.sendLiveRoomMsg(ImTargetType.GROUP, msg.getSenderUserId(), msg.getReceiverUserId(), liveGiftCommentMsg);
            }

            // 广播礼物判断
            if (BoolIntVal.True == giftDO.getIsNotice()) {
                LiveGiftNoticeMsg liveGiftNoticeMsg = new LiveGiftNoticeMsg();
                liveGiftNoticeMsg.setGiftId(msg.getGiftId());
                liveGiftNoticeMsg.setGiftName(giftDO.getGiftName());
                liveGiftNoticeMsg.setQuantity(msg.getQuantity());
                liveGiftNoticeMsg.setAnchorUserId(msg.getReceiverUserId());
                liveGiftNoticeMsg.setAnchorUserName(Optional.ofNullable(userService.getCard(msg.getReceiverUserId())).map(t -> t.getUserName()).orElse("神秘主播"));
                imService.sendLiveRoomMsg(ImTargetType.ALL, msg.getSenderUserId(), msg.getReceiverUserId(), liveGiftNoticeMsg);
            }

            // PK 条
            battleProvider.addValueAsync(msg.getReceiverUserId(), msg.getSenderUserId(), giftDO.getBattleValue() * msg.getQuantity());

            // 中奖横幅
            if (GiftTypeEnum.LUCKY == GiftTypeEnum.toEnum(giftDO.getGiftType())) {
                // 判断最大中奖倍数
                String luckyNoticeMultiples = giftDO.getLuckyNoticeMultiples();
                Integer maxMultiple = Optional.ofNullable(msg.getWinMultipleMax()).orElse(0);
                if (StringUtils.isNotBlank(luckyNoticeMultiples) && luckyNoticeMultiples.contains(String.format(",%d,", maxMultiple))) {
                    LiveLotteryNoticeMsg liveLotteryNotice = new LiveLotteryNoticeMsg();
                    liveLotteryNotice.setGiftId(giftDO.getGiftId());
                    liveLotteryNotice.setGiftName(giftDO.getGiftName());
                    liveLotteryNotice.setGiftType(giftDO.getGiftType());
                    liveLotteryNotice.setNoticeIcon(giftDO.getGiftIcon());
                    liveLotteryNotice.setMultiple(maxMultiple);
                    liveLotteryNotice.setWinAmount(msg.getWinAmount());
                    liveLotteryNotice.setAnchorUserId(msg.getReceiverUserId());
                    imService.sendLiveRoomMsg(ImTargetType.ALL, msg.getSenderUserId(), null, liveLotteryNotice);
                }
            }
        } else if (GiftSceneTypeEnum.SingleChat == giftSceneType) {
            // 发送礼物消息到房间
            LiveGiftComboMsg liveGiftComboMsg = new LiveGiftComboMsg();
            liveGiftComboMsg.setGiftId(msg.getGiftId());
            liveGiftComboMsg.setGiftName(giftDO.getGiftName());
            liveGiftComboMsg.setQuantity(msg.getQuantity());
            liveGiftComboMsg.setCombo(combo);
            liveGiftComboMsg.setComboKeyHash(SecureUtil.md5(comboKey));
            liveGiftComboMsg.setWinAmount(msg.getWinAmount());
            liveGiftComboMsg.setWinMultipleMax(msg.getWinMultipleMax());
            liveGiftComboMsg.setSourceType(msg.getSourceType());
            imService.sendChatGroupMsg(ImTargetType.GROUP, msg.getSenderUserId(), msg.getReceiverUserId(), msg.getChannelName(), liveGiftComboMsg);

            // 礼物弹幕
            if (liveGiftCommentMsg != null) {
                imService.sendChatGroupMsg(ImTargetType.GROUP, msg.getSenderUserId(), msg.getReceiverUserId(), msg.getChannelName(), liveGiftCommentMsg);
            }
        } else if (GiftSceneTypeEnum.VoiceChat == giftSceneType) {
            // 发送礼物消息到房间
            LiveGiftComboMsg liveGiftComboMsg = new LiveGiftComboMsg();
            liveGiftComboMsg.setGiftId(msg.getGiftId());
            liveGiftComboMsg.setGiftName(giftDO.getGiftName());
            liveGiftComboMsg.setQuantity(msg.getQuantity());
            liveGiftComboMsg.setCombo(combo);
            liveGiftComboMsg.setComboKeyHash(SecureUtil.md5(comboKey));
            liveGiftComboMsg.setWinAmount(msg.getWinAmount());
            liveGiftComboMsg.setWinMultipleMax(msg.getWinMultipleMax());
            liveGiftComboMsg.setSourceType(msg.getSourceType());
            imService.sendChatGroupMsg(ImTargetType.GROUP, msg.getSenderUserId(), msg.getReceiverUserId(), msg.getChannelName(), liveGiftComboMsg);

            // 礼物弹幕
            if (liveGiftCommentMsg != null) {
                imService.sendChatGroupMsg(ImTargetType.GROUP, msg.getSenderUserId(), msg.getReceiverUserId(), msg.getChannelName(), liveGiftCommentMsg);
            }
        }

        if (giftDO.getGiftType().equals(GiftTypeEnum.LOTTERY.getValue())) {
            // 抽奖礼物
            String lotteryId = giftDO.getLotteryId();
            if (StringUtils.isNotEmpty(lotteryId)) {
                lotteryActivityService.lottery(lotteryId, msg.getSenderUserId(), msg.getQuantity(), msg.getChannelId(), msg.getGiftSceneType(), msg.getChannelName(), msg.getReceiverUserId());
            }
        }
    }

    /**
     * 礼物连击超时
     *
     * @param msg
     */
    public void handleGiftComboTimeout(QueueGiftComboTimeoutMsg msg) {
        // 查询礼物消息
        GiftDO giftDO = giftService.getGift(msg.getGiftId(), null);

        // 弹幕礼物策略
        if (BoolIntVal.True == Optional.ofNullable(giftDO.getIsComment()).orElse(BoolIntVal.False) && msg.getCombo() > 1) {
            Integer commentComboMultiple = Optional.ofNullable(giftDO.getCommentComboMultiple()).orElse(0);
            if (commentComboMultiple > 0 && msg.getCombo() % commentComboMultiple != 0) {
                LiveGiftCommentMsg liveGiftCommentMsg = new LiveGiftCommentMsg();
                liveGiftCommentMsg.setGiftId(msg.getGiftId());
                liveGiftCommentMsg.setGiftName(giftDO.getGiftName());
                liveGiftCommentMsg.setQuantity(msg.getQuantity());
                liveGiftCommentMsg.setCombo(msg.getCombo());

                String comboKey = RedisCacheKeys.getGiftComboKey(msg.getReceiverUserId(), msg.getSenderUserId(), msg.getGiftId(), msg.getQuantity());
                Object channelName = cacheService.hashGet(RedisCacheKeys.GIFT_COMBO_CHAT_GROUP, comboKey);
                if (channelName != null) {
                    cacheService.hashDel(RedisCacheKeys.GIFT_COMBO_CHAT_GROUP, comboKey);
                    imService.sendChatGroupMsg(ImTargetType.GROUP, msg.getSenderUserId(), msg.getReceiverUserId(), channelName.toString(), liveGiftCommentMsg);
                } else {
                    imService.sendLiveRoomMsg(ImTargetType.GROUP, msg.getSenderUserId(), msg.getReceiverUserId(), liveGiftCommentMsg);
                }
            }
        }
    }

    /**
     * 进入直播间
     *
     * @param msg
     * @return
     */
    public void handleLiveUserJoin(QueueLiveUserJoinMsg msg) {
        Long anchorUserId = msg.getAnchorUserId();
        Long joinUserId = msg.getJoinUserId();

        // 缓存直播间资料
        userService.cacheLiveUserProfile(joinUserId, anchorUserId);

        if (anchorUserId.equals(joinUserId)) {
            // 主播自己
            return;
        }

        // 热度处理
        liveHeatProvider.changeLiveHeat(anchorUserId, 100, 0);

        // 缓存直播间人数
        livePopularityProvider.increase(anchorUserId, 1L);

        // 发送进入直播间消息
        Long curTime = msg.getTimestamp();
        if (cacheService.setIfAbsent(RedisLiveKeys.getLiveUserJoin(anchorUserId, joinUserId), String.valueOf(curTime), TimeConstants.SECOND_15_MILLS, TimeUnit.MILLISECONDS)) {
            LiveUserJoinMsg liveUserJoinMsg = new LiveUserJoinMsg();
            liveUserJoinMsg.setMounts(mountsService.getUserEnabledMounts(joinUserId));
            imService.sendLiveRoomMsg(ImTargetType.GROUP, joinUserId, anchorUserId, liveUserJoinMsg);
        }

        // PK 状态判定
        battleProvider.goBattleScene(anchorUserId, joinUserId);

        // 写入进房记录
        String hisId = SecureUtil.md5(String.format("%d:%d:%d:%d", joinUserId, anchorUserId, BoolIntVal.True, curTime));
        LiveUserJoinHisDO liveUserJoinHisDO = new LiveUserJoinHisDO();
        liveUserJoinHisDO.setHisId(hisId);
        liveUserJoinHisDO.setAnchorUserId(anchorUserId);
        liveUserJoinHisDO.setUserId(joinUserId);
        liveUserJoinHisDO.setChannelId(msg.getChannelId());
        liveUserJoinHisDO.setJoinType(BoolIntVal.True);
        liveUserJoinHisDO.setHisCreateTime(curTime);
        liveUserJoinHisDAO.insert(liveUserJoinHisDO);

        // 写入观看记录
        LiveStayHisDO liveStayHisDO = new LiveStayHisDO();
        liveStayHisDO.setHisId(hisId);
        liveStayHisDO.setJoinTime(curTime);
        liveStayHisDO.setUserId(joinUserId);
        liveStayHisDO.setAnchorUserId(anchorUserId);
        liveStayHisDAO.insert(liveStayHisDO);
    }

    /**
     * 离开直播间
     *
     * @param msg
     * @return
     */
    public void handleLiveUserExit(QueueLiveUserExitMsg msg) {
        Long anchorUserId = msg.getAnchorUserId();
        Long exitUserId = msg.getExitUserId();

        // 移除直播间资料
        if (userService.removeLiveUserProfile(msg.getExitUserId(), msg.getAnchorUserId())) {
            if (anchorUserId.equals(exitUserId)) {
                // 主播自己
                return;
            }

            // 热度处理
            liveHeatProvider.changeLiveHeat(anchorUserId, -100, 0);

            // 缓存直播间人数
            livePopularityProvider.decrease(anchorUserId, 1L);

            // 写入出房记录
            Long curTime = msg.getTimestamp();
            String hisId = SecureUtil.md5(String.format("%d:%d:%d:%d", exitUserId, anchorUserId, BoolIntVal.True, curTime));
            LiveUserJoinHisDO liveUserJoinHisDO = new LiveUserJoinHisDO();
            liveUserJoinHisDO.setHisId(hisId);
            liveUserJoinHisDO.setAnchorUserId(anchorUserId);
            liveUserJoinHisDO.setUserId(exitUserId);
            liveUserJoinHisDO.setChannelId(msg.getChannelId());
            liveUserJoinHisDO.setJoinType(BoolIntVal.False);
            liveUserJoinHisDO.setHisCreateTime(curTime);
            liveUserJoinHisDAO.insert(liveUserJoinHisDO);

            // 更新进房记录
            LiveStayHisDO lastHis = liveStayHisDAO.selectLast(exitUserId, anchorUserId);
            if (lastHis != null) {
                lastHis.setExitTime(System.currentTimeMillis());
                Long duration = lastHis.getExitTime() - lastHis.getJoinTime();
                lastHis.setStayDuration(duration);
                liveStayHisDAO.update(lastHis);

                // 加入统计
                statPlatformService.addPlatformStat(msg.getChannelId(), StatPlatformTypeEnum.LIVE_STAY_DURATION, BigDecimal.valueOf(duration / 1000));
            }
        }
    }

    /**
     * 开播
     *
     * @param msg
     */
    public void handleLiveStart(QueueLiveStartMsg msg) {

        AnchorInfoVO info = anchorService.getInfo(msg.getAnchorUserId());
        if (info != null) {
            // 更新经纬度
            userService.setPosition(msg.getAnchorUserId(), info.getLatitude(), info.getLongitude(), info.getAreaId());
            // 更新直播状态
            userSearchService.setLiveStatus(msg.getAnchorUserId(), LiveStatusEnum.Living);
        }

        // 发送直播间
        LiveStatusLiveMsg liveStatusLiveMsg = new LiveStatusLiveMsg();
        liveStatusLiveMsg.setLiveStatus(LiveStatusEnum.Living.getValue());
        imService.sendLiveRoomMsg(ImTargetType.GROUP, null, msg.getAnchorUserId(), liveStatusLiveMsg);

        // 发送粉丝
        List<Long> fansUserIds = socialRelationService.getFansUserIds(msg.getAnchorUserId());
        if (CollectionUtil.isNotEmpty(fansUserIds)) {

            PushTaskDO taskDO = new PushTaskDO();
            taskDO.setTaskId(identityService.getId());
            taskDO.setTaskName("系统消息-主播开播");
            taskDO.setTaskStatus(TaskStatusEnum.WAIT.getValue());
            taskDO.setTaskCreateTime(System.currentTimeMillis());
            taskDO.setTaskType(TaskTypeEnum.BATCH_PUSH.getValue());
            taskDO.setTargetValue(CollectionUtil.join(fansUserIds, ","));
            taskDO.setAdminId(0L);
            String pushTitle = String.format("您关注的主播%s开播啦，点击来撩~", userService.getUserName(msg.getAnchorUserId()));
            taskDO.setPushTitle(pushTitle);
            taskDO.setPushBody(info == null ? "现场发福利啦，快来围观~" : info.getLiveTitle());
            taskDO.setRemark("系统自动推送");

            PushMsgParam pushParam = new PushMsgParam();
            pushParam.setActionType(PushActionType.JUMP_NATIVE.getValue());
            pushParam.setModuleType(ModuleTypeEnum.LIVE_ROOM.getDesc());
            pushParam.setModuleId(Convert.toStr(msg.getAnchorUserId()));

            taskDO.setExtParameters(JSON.toJSONString(pushParam));
            pushService.addPushTask(taskDO);
        }
    }

    /**
     * 下播
     *
     * @param msg
     */
    public void handleLiveStop(QueueLiveStopMsg msg) {

        // 更新直播状态
        userSearchService.setLiveStatus(msg.getAnchorUserId(), LiveStatusEnum.Closed);

        // 发送直播间
        LiveStatusLiveMsg liveStatusLiveMsg = new LiveStatusLiveMsg();
        liveStatusLiveMsg.setLiveStatus(LiveStatusEnum.Closed.getValue());
        imService.sendLiveRoomMsg(ImTargetType.GROUP, null, msg.getAnchorUserId(), liveStatusLiveMsg);

        // 如果有 PK 结束 PK
        battleProvider.battleOverByUserId(msg.getAnchorUserId());
    }

    /**
     * 推流
     *
     * @param msg
     */
    public void handleStreamPush(QueueLiveStreamPushMsg msg) {
        // 发送直播间
        LiveStatusStreamMsg liveStatusStreamMsg = new LiveStatusStreamMsg();
        liveStatusStreamMsg.setStreamStatus(StreamStatusEnum.Pushing.getValue());
        liveStatusStreamMsg.setPullUrl(qiniuLiveProvider.getPullUrl(msg.getAnchorUserId(), 300));
        imService.sendLiveRoomMsg(ImTargetType.GROUP, null, msg.getAnchorUserId(), liveStatusStreamMsg);
    }

    /**
     * 断流
     *
     * @param msg
     */
    public void handleStreamClose(QueueLiveStreamCloseMsg msg) {
        // 发送直播间
        LiveStatusStreamMsg liveStatusStreamMsg = new LiveStatusStreamMsg();
        liveStatusStreamMsg.setStreamStatus(StreamStatusEnum.Closed.getValue());
        imService.sendLiveRoomMsg(ImTargetType.GROUP, null, msg.getAnchorUserId(), liveStatusStreamMsg);

        // 保存回放
        String vodUrl = qiniuLiveProvider.saveStream(msg.getAnchorUserId(), msg.getStartTime(), msg.getStopTime());
        liveStreamHisDAO.updateVodUrl(msg.getLiveHisId(), vodUrl);
    }

    /**
     * 用户进入单聊房间
     *
     * @param msg
     */
    public void handleSingleChatUserJoin(QueueSingleChatUserJoinMsg msg) {
        // 缓存资料
        userService.cacheChatUserProfile(msg.getJoinUserId(), msg.getChannelName());
        SingleChatHisDO singleChatHis = singleChatHisDAO.selectById(msg.getChannelName());
        if (singleChatHis != null && singleChatHis.getUserId().equals(msg.getJoinUserId())) {
            long minutesRemain = walletService.getBalance(singleChatHis.getUserId(), singleChatHis.getHisPriceType()) / singleChatHis.getHisPrice();
            minutesRemain += walletService.getChatTimeWalletMinutes(singleChatHis.getUserId(), ChatTimeWalletType.SINGLE_CHAT);
            // 发送余额提醒
            SingleChatBalanceRemindMsg singleChatBalanceRemindMsg = new SingleChatBalanceRemindMsg();
            singleChatBalanceRemindMsg.setHisId(singleChatHis.getHisId());
            singleChatBalanceRemindMsg.setMinutesRemain(minutesRemain);
            imService.sendChatGroupMsg(ImTargetType.GROUP, null, null, singleChatHis.getHisId(), singleChatBalanceRemindMsg);
        }
    }

    /**
     * 用户离开单聊房间
     *
     * @param msg
     */
    public void handleSingleChatUserExit(QueueSingleChatUserExitMsg msg) {
        // 移除资料
        userService.removeChatUserProfile(msg.getExitUserId(), msg.getChannelName());
    }

    /**
     * 用户加入聊天频道
     *
     * @param msg
     */
    public void handleChatUserJoin(QueueChatUserJoinMsg msg) {
        // 查询聊天室是否存在
    }

    /**
     * 用户离开聊天频道
     *
     * @param msg
     */
    public void handleChatUserExit(QueueChatUserExitMsg msg) {
        SingleChatHisDO singleChatHis = singleChatHisDAO.selectById(msg.getChannelName());
        if (singleChatHis != null) {
            if (ChatStatusEnum.Calling == ChatStatusEnum.toEnum(singleChatHis.getHisStatus())) {
                SingleChatStopReasonEnum singleChatStopReason = null;
                if (singleChatHis.getUserId().equals(msg.getExitUserId())) {
                    singleChatStopReason = SingleChatStopReasonEnum.UserClose;
                } else if (singleChatHis.getTargetUserId().equals(msg.getExitUserId())) {
                    singleChatStopReason = SingleChatStopReasonEnum.AnchorClose;
                }

                if (singleChatStopReason != null) {
                    // 中止
                    singleChatProvider.stopChat(singleChatHis.getHisId(), singleChatStopReason, msg.getAppVer());
                }
            }
            return;
        }

        VoiceChatHisDO hisDO = voiceChatHisDAO.selectById(Convert.toLong(msg.getChannelName()));
        if (hisDO != null) {
            if (ChatStatusEnum.Calling == ChatStatusEnum.toEnum(hisDO.getHisStatus())) {
                VoiceChatStopReasonEnum stopReasonEnum = null;
                if (hisDO.getUserId().equals(msg.getExitUserId())) {
                    stopReasonEnum = VoiceChatStopReasonEnum.UserLeave;
                } else if (hisDO.getTargetUserId().equals(msg.getExitUserId())) {
                    stopReasonEnum = VoiceChatStopReasonEnum.TargetUserLeave;
                }

                if (stopReasonEnum != null) {
                    // 中止
                    log.info("声网回调通知，{}", stopReasonEnum.getDesc());
                    voiceChatService.stopChat(hisDO.getHisId(), stopReasonEnum, System.currentTimeMillis(), 0, msg.getAppVer());
                }
            }
            return;
        }
    }

    /**
     * 用户在线状态改变
     *
     * @param msg
     */
    public void handleUserOnlineChange(QueueUserOnlineChangeMsg msg) {
        if (BoolIntVal.True == msg.getOnlineStatus()) {
            // 尝试领取会员免费时长
            vipService.getDayFreeMinutes(msg.getUserId());
//            if (chatMatchProvider.addMatchPoolUser(msg.getUserId())) {
//                chatMatchProvider.matchDelay(msg.getUserId(), TimeConstants.SECOND_15_MILLS);
//            } else {
//                chatMatchProvider.addMatchPoolAnchor(msg.getUserId());
//            }
        } else {
            chatMatchProvider.removeMatchPoolUser(msg.getUserId());
            chatMatchProvider.removeMatchPoolAnchor(msg.getUserId());

            // 判断是否在直播间里
            Long anchorUserId = userService.getLiveUserRoom(msg.getUserId());
            if (anchorUserId > 0) {
                QueueLiveUserExitMsg liveUserExitMsg = new QueueLiveUserExitMsg();
                liveUserExitMsg.setChannelId(msg.getChannelId());
                liveUserExitMsg.setExitUserId(msg.getUserId());
                liveUserExitMsg.setAnchorUserId(anchorUserId);
                handleLiveUserExit(liveUserExitMsg);
            }
        }
    }

    /**
     * 语音速配请求
     *
     * @param msg
     */
    public void handleVoiceMatchRequest(QueueVoiceMatchRequestMsg msg) {
        Long hisId = msg.getHisId();
        Long channelId = msg.getChannelId();
        Long userId = msg.getUserId();
        Integer userSex = msg.getUserSex();
        long matchStartTime = msg.getMatchStartTime();
        if (!voiceChatProvider.hasFreeTime(userId, matchStartTime) && walletService.getBalance(userId, WalletTypeEnum.STAR_DIAMOND) < 100L) {
            log.info("余额不足，请先充值。hisId {}", hisId);
            VoiceChatMatchFailMsg matchMsg = new VoiceChatMatchFailMsg();
            matchMsg.setHisId(hisId);
            matchMsg.setUserId(userId);
            matchMsg.setTargetUserId(null);
            matchMsg.setMsg("余额不足，请先充值");
            imService.sendCustomImMsg(null, userId, matchMsg, null);
            userService.updateVoiceChatStatus(userId, VoiceChatStatus.IDLE);
            return;
        }

        // 匹配应该截止时间
        long matchShouldEndTime = matchStartTime + 60 * 1000L;

        log.info("hisId {} 开始匹配", hisId);
        // 排除当前用户
        redisService.hashPut(RedisCacheKeys.getVoiceChatExcludedUser(hisId), Convert.toStr(userId), Convert.toStr(System.currentTimeMillis()));
        // 匹配用户
        log.info("语聊速配匹配用户开始");
        List<Long> matchUserIds = voiceChatProvider.matchUser(hisId, userSex, matchShouldEndTime, MatchModeEnum.ACTIVE.getValue(), voiceChatService.getMatchCount());
        if (CollectionUtil.isEmpty(matchUserIds)) {
            log.info("超时未匹配到用户,hisId {}", hisId);
            userService.updateVoiceChatStatus(userId, VoiceChatStatus.IDLE);
            return;
        }

        // 匹配成功
        matchSuccess(hisId, channelId, userId, matchStartTime, matchUserIds, 1);
        voiceChatHisDAO.updateMatchUserIds(hisId, CollectionUtil.join(matchUserIds, ","));
    }

    /**
     * 匹配成功
     *
     * @param hisId
     * @param channelId
     * @param userId
     * @param matchStartTime
     * @param matchUserIds
     * @param round
     */
    private void matchSuccess(Long hisId, Long channelId, Long userId, Long matchStartTime, List<Long> matchUserIds, int round) {
        log.info("匹配到用户 hisId {}, matchUserIds {},等待对方同意", hisId, matchUserIds);
        long matchEndTime = System.currentTimeMillis();
        // 更新
        VoiceChatHisDO hisUpdate = new VoiceChatHisDO();
        hisUpdate.setHisId(hisId);
        hisUpdate.setMatchEndTime(matchEndTime);
        hisUpdate.setHisUpdateTime(matchEndTime);
        voiceChatHisDAO.updateById(hisUpdate);

        UserCardVO userCard = userService.getCard(userId);
        Long[] ids = identityService.getIds(matchUserIds.size());
        voiceChatProvider.addRoundUsers(hisId, round, matchUserIds);

        for (int i = 0; i < matchUserIds.size(); i++) {
            Long matchUserId = matchUserIds.get(i);
            // 更新用户聊天状态
            userService.updateVoiceChatStatus(matchUserId, VoiceChatStatus.BUSY);
            // 增加匹配次数
            familyService.incrementMatchTimes(matchUserId);

            VoiceChatMatchWaitMsg matchMsgToUser = new VoiceChatMatchWaitMsg();
            matchMsgToUser.setHisId(hisId);
            matchMsgToUser.setUserId(userId);
            matchMsgToUser.setTargetUserId(matchUserId);
            matchMsgToUser.setRound(round);
            matchMsgToUser.setMsg("匹配到用户，等待对方同意");
            matchMsgToUser.setMatchStartTime(matchStartTime);

            UserCardVO targetUserCard = userService.getCard(matchUserId);
            matchMsgToUser.setUserName(Optional.ofNullable(userCard).map(UserCardVO::getUserName).orElse(""));
            matchMsgToUser.setUserAvatar(Optional.ofNullable(userCard).map(UserCardVO::getUserAvatar).orElse(""));
            matchMsgToUser.setTargetUserName(Optional.ofNullable(targetUserCard).map(UserCardVO::getUserName).orElse(""));
            matchMsgToUser.setTargetUserAvatar(Optional.ofNullable(targetUserCard).map(UserCardVO::getUserAvatar).orElse(""));
            imService.sendCustomImMsg(null, userId, matchMsgToUser, null);

            VoiceChatMatchWaitMsg matchMsgToTargetUser = new VoiceChatMatchWaitMsg();
            matchMsgToTargetUser.setHisId(hisId);
            matchMsgToTargetUser.setUserId(userId);
            matchMsgToTargetUser.setTargetUserId(matchUserId);
            matchMsgToTargetUser.setRound(round);
            matchMsgToTargetUser.setMsg("语音速配，请同意");
            matchMsgToTargetUser.setMatchStartTime(matchStartTime);
            matchMsgToTargetUser.setUserName(Optional.ofNullable(userCard).map(UserCardVO::getUserName).orElse(""));
            matchMsgToTargetUser.setUserAvatar(Optional.ofNullable(userCard).map(UserCardVO::getUserAvatar).orElse(""));
            matchMsgToTargetUser.setTargetUserName(Optional.ofNullable(targetUserCard).map(UserCardVO::getUserName).orElse(""));
            matchMsgToTargetUser.setTargetUserAvatar(Optional.ofNullable(targetUserCard).map(UserCardVO::getUserAvatar).orElse(""));
            imService.sendCustomImMsg(null, matchUserId, matchMsgToTargetUser, null);

            // 10秒钟检测
            QueueVoiceMatchWaitTimeoutMsg waitTimeoutMsg = new QueueVoiceMatchWaitTimeoutMsg();
            waitTimeoutMsg.setHisId(hisId);
            waitTimeoutMsg.setCurTargetUserId(matchUserId);
            waitTimeoutMsg.setRound(round);
            waitTimeoutMsg.setMatchStartTime(matchStartTime);
            waitTimeoutMsg.setCurEndTime(matchEndTime + 10 * 1000);
            rocketProducer.sendMsg(RocketTopic.TPC_DELAY, RocketTag.VOICE_MATCH_WAIT_TIMEOUT, waitTimeoutMsg.serialize(), waitTimeoutMsg.getCurEndTime());

            // 统计
            statPlatformService.addPlatformStat(channelId, StatPlatformTypeEnum.VOICE_CHAT_TIMES_MATCH_REQUEST, BigDecimal.ONE);

            // 消息通知
            PushTaskDO taskDO = new PushTaskDO();
            taskDO.setTaskId(ids[i]);
            taskDO.setTaskName("系统消息-语音速配匹配到用户");
            taskDO.setTaskStatus(TaskStatusEnum.WAIT.getValue());
            taskDO.setTaskCreateTime(System.currentTimeMillis());
            taskDO.setTaskType(TaskTypeEnum.SINGLE_PUSH.getValue());
            taskDO.setTargetValue(Convert.toStr(matchUserId));
            taskDO.setAdminId(0L);
            taskDO.setPushTitle(String.format("%s 太幸运了！%s 通过「语音速配」", userService.getUserName(matchUserId), userService.getUserName(userId)));
            taskDO.setPushBody("找到了你，快来同意吧>>");
            taskDO.setRemark("系统自动推送");

            pushService.addPushTask(taskDO);
        }

    }

    /**
     * 语音速配拒绝
     *
     * @param msg
     */
    public void handleVoiceChatReject(QueueVoiceChatRejectMsg msg) {
        log.info("语音速配拒绝 hisId {}, curTargetUserId {}, round {}", msg.getHisId(), msg.getTargetUserId(), msg.getRound());

        Long hisId = msg.getHisId();
        VoiceChatHisDO hisDO = voiceChatHisDAO.selectById(hisId);
        if (hisDO == null) {
            log.info("未找到记录{}", hisId);
            return;
        }

        ChatStatusEnum statusEnum = ChatStatusEnum.toEnum(hisDO.getHisStatus());
        if (statusEnum != ChatStatusEnum.Ringing) {
            log.info("hisId {}，status {}，只有呼叫中才可拒绝", hisId, statusEnum.getDesc());
            return;
        }

        Long userId = msg.getUserId();
        Long targetUserId = msg.getTargetUserId();
        Long matchStartTime = msg.getMatchStartTime();
        long timestamp = msg.getTimestamp();
        int round = msg.getRound();

        // 当前轮次
        int curRound = voiceChatProvider.getCurRound(hisId);
        log.info("当前轮次 {} , 处理第{}轮请求", curRound, round);
        if (curRound > round) {
            log.info("本轮次已过期");
            return;
        }
        // 处理拒绝
        voiceChatProvider.addExcludedUserIds(hisId, targetUserId);
        // 拒绝人数+1
        Integer rejectCount = Convert.toInt(redisService.hashIncreaseLong(RedisChatKeys.getVoiceChatRoundReject(hisId), Convert.toStr(round), 1));
        // 如果本轮次所有人都拒绝
        List<Long> roundUsers = voiceChatProvider.getRoundUserIds(hisId, round);
        if (rejectCount >= roundUsers.size()) {
            // 且本轮次有剩余匹配时间  =>  发起下一轮次，继续匹配
            if (System.currentTimeMillis() < msg.getMatchShouldEndTime()) {
                log.info("hisId {} ，{}人拒绝了，系统自动发起下一轮", hisId, rejectCount);
                round += 1;
                List<Long> matchUserIds = voiceChatProvider.matchUser(hisId, hisDO.getUserSex(), msg.getMatchShouldEndTime(), MatchModeEnum.ACTIVE.getValue(), voiceChatService.getMatchCount());
                if (CollectionUtil.isEmpty(matchUserIds)) {
                    log.info("未匹配到用户,{}", hisId);
                    userService.updateVoiceChatStatus(userId, VoiceChatStatus.IDLE);
                    return;
                }
                // 匹配成功
                matchSuccess(hisId, hisDO.getChannelId(), userId, matchStartTime, matchUserIds, round);
                voiceChatHisDAO.updateMatchUserIds(hisId, CollectionUtil.join(voiceChatProvider.getExcludedUserIds(hisId), ","));
            } else {
                log.info("hisId {} ，{}人拒绝了，没有剩余时间", hisId, round, rejectCount);
                // 再次判断状态
                ChatStatusEnum status = ChatStatusEnum.toEnum(hisDO.getHisStatus());
                if (status != ChatStatusEnum.Ringing) {
                    log.info("hisId{}，状态{},只有呼叫中才可拒绝", hisId, status.getDesc());
                    return;
                }

                // 并且没有剩余匹配时间（1分钟）
                // 更新数据库 => 拒绝本次语音速配请求
                VoiceChatHisDO updateHis = new VoiceChatHisDO();
                updateHis.setHisStatus(ChatStatusEnum.Rejected.getValue());
                updateHis.setHisId(hisId);
                updateHis.setHisUpdateTime(timestamp);
                voiceChatHisDAO.updateById(updateHis);

                // 更新语音状态
                userService.updateVoiceChatStatus(userId, VoiceChatStatus.IDLE);

                VoiceChatRejectMsg rejectMsgToUser = new VoiceChatRejectMsg();
                rejectMsgToUser.setHisId(hisId);
                rejectMsgToUser.setUserId(userId);
                rejectMsgToUser.setTargetUserId(targetUserId);
                rejectMsgToUser.setMatchStartTime(matchStartTime);
                rejectMsgToUser.setMsg("对方拒绝语音邀请");
                imService.sendCustomImMsg(null, userId, rejectMsgToUser, null);
            }
        }
    }

    /**
     * 语音速配开始
     *
     * @param msg
     */
    public void handleVoiceChatStart(QueueVoiceChatStartMsg msg) {
        log.info("语音速配开始 {}", msg.getHisId());

        Long hisId = msg.getHisId();
        Long hisStartTime = msg.getHisStartTime();

        Long userId = msg.getUserId();
        Long targetUserId = msg.getTargetUserId();
        Long channelId = msg.getChannelId();
        String appVer = msg.getAppVer();

        String userChannelName = providerBuilder.provider(appVer).voiceChatChannelName(hisId, userId);
        String targetChannelName = providerBuilder.provider(appVer).voiceChatChannelName(hisId, targetUserId);

        // 获取 token
        String targetUserToken = providerBuilder.provider(appVer).buildToken(targetChannelName, targetUserId, AgoraProvider.Role.Role_Publisher.initValue, 0);
        String userToken = providerBuilder.provider(appVer).buildToken(userChannelName, userId, AgoraProvider.Role.Role_Subscriber.initValue, 0);

        // 匹配成功im
        VoiceChatMatchSuccessMsg successMsgToUser = new VoiceChatMatchSuccessMsg();
        successMsgToUser.setUserId(userId);
        successMsgToUser.setTargetUserId(targetUserId);
        successMsgToUser.setHisId(hisId);
        successMsgToUser.setHisStartTime(hisStartTime);
        successMsgToUser.setUserCard(userService.getCard(userId));
        successMsgToUser.setTargetUserCard(userService.getCard(targetUserId));
        successMsgToUser.setUserToken(userToken);
        successMsgToUser.setTargetUserToken(targetUserToken);
        successMsgToUser.setChannelName(userChannelName);
        successMsgToUser.setMsg("马上开撩");
        imService.sendCustomImMsg(null, userId, successMsgToUser, null);

        VoiceChatMatchSuccessMsg successMsgToTargetUser = new VoiceChatMatchSuccessMsg();
        successMsgToTargetUser.setUserId(userId);
        successMsgToTargetUser.setTargetUserId(targetUserId);
        successMsgToTargetUser.setHisId(hisId);
        successMsgToTargetUser.setHisStartTime(hisStartTime);
        successMsgToTargetUser.setUserCard(userService.getCard(userId));
        successMsgToTargetUser.setTargetUserCard(userService.getCard(targetUserId));
        successMsgToTargetUser.setUserToken(userToken);
        successMsgToTargetUser.setTargetUserToken(targetUserToken);
        successMsgToTargetUser.setChannelName(targetChannelName);
        successMsgToTargetUser.setMsg("马上开撩");
        imService.sendCustomImMsg(null, targetUserId, successMsgToTargetUser, null);

        // 统计
        statPlatformService.addPlatformStat(channelId, StatPlatformTypeEnum.VOICE_CHAT_TIMES_MATCH_CONNECT, BigDecimal.ONE);
    }

    /**
     * 语音速配停止
     *
     * @param msg
     */
    public void handleVoiceChatStop(QueueVoiceChatStopMsg msg) {
        log.info("语音速配停止 {}", msg.getHisId());

        Long hisId = msg.getHisId();
        VoiceChatHisDO hisDO = voiceChatHisDAO.selectById(hisId);
        if (hisDO == null) {
            log.info("未找到记录{}", hisId);
            return;
        }

        Long userId = hisDO.getUserId();
        Long targetUserId = hisDO.getTargetUserId();
        Long hisPrice = hisDO.getHisPrice();
        Integer hisPriceType = hisDO.getHisPriceType();
        String hisResourceId = hisDO.getHisResourceId();
        String hisReplayId = hisDO.getHisReplayId();
        int hisMinutes = msg.getHisMinutes();
        Long hisStartTime = hisDO.getHisStartTime();
        Long hisStopTime = msg.getHisStopTime();

        VoiceChatStopReasonEnum reasonEnum = msg.getReasonEnum();

        // 停止录制
        String hisReplay = providerBuilder.provider(msg.getAppVer()).stopRecord(Convert.toStr(hisId), hisResourceId, hisReplayId);
        if (StringUtils.isNotBlank(hisReplay)) {
            voiceChatHisDAO.updateReplay(hisId, hisReplay);
        }

        // 金币分钟
        Integer hisTimeMinutes = Optional.ofNullable(hisDO.getHisTimeMinutes()).orElse(0);
        Integer hisCoinMinutes = hisMinutes > 0 ? hisMinutes - hisTimeMinutes : 0;

        // 中止频道
        providerBuilder.provider(msg.getAppVer()).disableChannel(Convert.toStr(hisId));

        // 发送停止消息
        VoiceChatStopMsg voiceChatStopMsg = new VoiceChatStopMsg();
        voiceChatStopMsg.setHisId(hisId);
        voiceChatStopMsg.setUserId(userId);
        voiceChatStopMsg.setTargetUserId(targetUserId);
        voiceChatStopMsg.setStopReason(reasonEnum.getValue());
        voiceChatStopMsg.setStopReasonName(reasonEnum.getDesc());
        voiceChatStopMsg.setHisMinutes(hisMinutes);
        voiceChatStopMsg.setHisCoinMinutes(hisCoinMinutes);
        voiceChatStopMsg.setHisPrice(hisPrice);
        voiceChatStopMsg.setHisAmount(hisCoinMinutes * hisPrice);
        voiceChatStopMsg.setHisTimeMinutes(hisTimeMinutes);
        voiceChatStopMsg.setHisAmountName(WalletTypeEnum.toEnum(hisPriceType).getName());
        voiceChatStopMsg.setHisStartTime(hisStartTime);
        voiceChatStopMsg.setHisStopTime(hisStopTime);
        voiceChatStopMsg.setMsg("已结束");
        imService.sendChatGroupMsg(ImTargetType.GROUP, null, null, Convert.toStr(hisId), voiceChatStopMsg);
        // 移除统计
        redisService.hashDel(RedisFinanceKeys.VOICE_CHAT_CHARGING, Convert.toStr(hisId));

        // 统计
        Long duration = hisStopTime - hisStartTime;
        statPlatformService.addPlatformStat(hisDO.getChannelId(), StatPlatformTypeEnum.VOICE_CHAT_CALL_DURATION,
                new BigDecimal(duration.toString()));
    }
}
