package com.starbuds.server.service.media.rocket.listener;

import com.alibaba.fastjson.JSON;
import com.aliyun.openservices.ons.api.Action;
import com.aliyun.openservices.ons.api.ConsumeContext;
import com.aliyun.openservices.ons.api.Message;
import com.aliyun.openservices.ons.api.MessageListener;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.pojo.mq.*;
import com.starbuds.server.service.media.rocket.handler.DelayMsgHandler;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * 延迟队列消息监听
 *
 * @author Lay
 * @date 2019/12/25
 */
@Component
@Slf4j
public class DelayMsgListener implements MessageListener {

    @Resource
    DelayMsgHandler delayMsgHandler;

    @Override
    public Action consume(Message message, ConsumeContext consumeContext) {
        try {
            log.info("收到延迟队列消息 {} {}", message.getTopic(), message.getTag());
            switch (message.getTag()) {
                case RocketTag.LIVE_HEAT_EXPIRE:
                    delayMsgHandler.handleLiveHeatExpire(JSON.parseObject(message.getBody(), QueueLiveHeatExpireMsg.class));
                    break;
                case RocketTag.LIVE_TIMEOUT:
                    delayMsgHandler.handleLiveTimeout(JSON.parseObject(message.getBody(), QueueLiveTimeoutMsg.class));
                    break;
                case RocketTag.SINGLE_CHAT_RING_TIMEOUT:
                    delayMsgHandler.handleSingleChatRingTimeout(JSON.parseObject(message.getBody(), QueueSingleChatRingTimeoutMsg.class));
                    break;
                case RocketTag.SINGLE_CHAT_CHARGING_TICK:
                    delayMsgHandler.handleSingleChatChargingTick(JSON.parseObject(message.getBody(), QueueSingleChatChargingTickMsg.class));
                    break;
                case RocketTag.SINGLE_CHAT_MATCH_DELAY:
                    delayMsgHandler.handleSingleChatMatchDelay(JSON.parseObject(message.getBody(), QueueSingleChatMatchDelayMsg.class));
                    break;
                case RocketTag.BATTLE_INVITE_TIMEOUT:
                    delayMsgHandler.handleBattleInviteTimeout(JSON.parseObject(message.getBody(), QueueBattleInviteTimeoutMsg.class));
                    break;
                case RocketTag.BATTLE_WAIT_TIMEOUT:
                    delayMsgHandler.handleBattleWaitTimeout(JSON.parseObject(message.getBody(), QueueBattleWaitTimeoutMsg.class));
                    break;
                case RocketTag.BATTLE_STAGE_DELAY:
                    delayMsgHandler.handleBattleStageDelay(JSON.parseObject(message.getBody(), QueueBattleStageDelayMsg.class));
                    break;
                case RocketTag.VOICE_MATCH_WAIT_TIMEOUT:
                    // 语音速配等待超时
                    delayMsgHandler.handleVoiceMatchWaitTimeout(JSON.parseObject(message.getBody(), QueueVoiceMatchWaitTimeoutMsg.class));
                    break;
                case RocketTag.VOICE_MATCH_TIMEOUT:
                    // 语音速配超时
                    delayMsgHandler.handleVoiceMatchTimeout(JSON.parseObject(message.getBody(), QueueVoiceMatchTimeoutMsg.class));
                    break;
                case RocketTag.VOICE_CHAT_CHARGING_TICK:
                    delayMsgHandler.handleVoiceChatChargingTick(JSON.parseObject(message.getBody(), QueueVoiceChatChargingTickMsg.class));
                    break;
                default:
                    break;
            }
            return Action.CommitMessage;
        } catch (Exception e) {
            log.error("消息处理失败", e);
            return Action.ReconsumeLater;
        }
    }
}
