package com.starbuds.server.service.media.rocket.listener;

import com.alibaba.fastjson.JSON;
import com.aliyun.openservices.ons.api.Action;
import com.aliyun.openservices.ons.api.ConsumeContext;
import com.aliyun.openservices.ons.api.Message;
import com.aliyun.openservices.ons.api.MessageListener;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.pojo.mq.*;
import com.starbuds.server.service.media.rocket.handler.NormalMsgHandler;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * 普通队列消息监听
 *
 * @author Lay
 * @date 2020/1/2
 */
@Component
@Slf4j
public class NormalMsgListener implements MessageListener {

    @Resource
    NormalMsgHandler normalMsgHandler;

    /**
     * 消息监听
     *
     * @param message
     * @param consumeContext
     * @return
     */
    @Override
    public Action consume(Message message, ConsumeContext consumeContext) {
        try {
            log.info("收到普通队列消息 {} {}", message.getTopic(), message.getTag());
            switch (message.getTag()) {
                case RocketTag.GIFT_PAID:
                    // 礼物支付
                    normalMsgHandler.handleGiftPaid(JSON.parseObject(message.getBody(), QueueGiftPaidMsg.class));
                    break;
                case RocketTag.GIFT_COMBO_TIMEOUT:
                    // 礼物连击超时
                    normalMsgHandler.handleGiftComboTimeout(JSON.parseObject(message.getBody(), QueueGiftComboTimeoutMsg.class));
                    break;
                case RocketTag.LIVE_USER_JOIN:
                    // 进入直播间
                    normalMsgHandler.handleLiveUserJoin(JSON.parseObject(message.getBody(), QueueLiveUserJoinMsg.class));
                    break;
                case RocketTag.LIVE_USER_EXIT:
                    // 离开直播间
                    normalMsgHandler.handleLiveUserExit(JSON.parseObject(message.getBody(), QueueLiveUserExitMsg.class));
                    break;
                case RocketTag.LIVE_START:
                    // 开播
                    normalMsgHandler.handleLiveStart(JSON.parseObject(message.getBody(), QueueLiveStartMsg.class));
                    break;
                case RocketTag.LIVE_STOP:
                    // 下播
                    normalMsgHandler.handleLiveStop(JSON.parseObject(message.getBody(), QueueLiveStopMsg.class));
                    break;
                case RocketTag.STREAM_PUSH:
                    // 推流
                    normalMsgHandler.handleStreamPush(JSON.parseObject(message.getBody(), QueueLiveStreamPushMsg.class));
                    break;
                case RocketTag.STREAM_CLOSE:
                    // 断流
                    normalMsgHandler.handleStreamClose(JSON.parseObject(message.getBody(), QueueLiveStreamCloseMsg.class));
                    break;
                case RocketTag.SINGLE_CHAT_USER_JOIN:
                    normalMsgHandler.handleSingleChatUserJoin(JSON.parseObject(message.getBody(), QueueSingleChatUserJoinMsg.class));
                    break;
                case RocketTag.SINGLE_CHAT_USER_EXIT:
                    normalMsgHandler.handleSingleChatUserExit(JSON.parseObject(message.getBody(), QueueSingleChatUserExitMsg.class));
                    break;
                case RocketTag.CHAT_USER_JOIN:
                    normalMsgHandler.handleChatUserJoin(JSON.parseObject(message.getBody(), QueueChatUserJoinMsg.class));
                    break;
                case RocketTag.CHAT_USER_EXIT:
                    normalMsgHandler.handleChatUserExit(JSON.parseObject(message.getBody(), QueueChatUserExitMsg.class));
                    break;
                case RocketTag.USER_ONLINE_CHANGE:
                    normalMsgHandler.handleUserOnlineChange(JSON.parseObject(message.getBody(), QueueUserOnlineChangeMsg.class));
                    break;
                case RocketTag.VOICE_MATCH_REQUEST:
                    // 发起语音速配
                    normalMsgHandler.handleVoiceMatchRequest(JSON.parseObject(message.getBody(), QueueVoiceMatchRequestMsg.class));
                    break;
                case RocketTag.VOICE_CHAT_REJECT:
                    // 语音速配拒绝
                    normalMsgHandler.handleVoiceChatReject(JSON.parseObject(message.getBody(), QueueVoiceChatRejectMsg.class));
                    break;
                case RocketTag.VOICE_CHAT_START:
                    // 语音速配开始
                    normalMsgHandler.handleVoiceChatStart(JSON.parseObject(message.getBody(), QueueVoiceChatStartMsg.class));
                    break;
                case RocketTag.VOICE_CHAT_STOP:
                    // 语音速配停止
                    normalMsgHandler.handleVoiceChatStop(JSON.parseObject(message.getBody(), QueueVoiceChatStopMsg.class));
                    break;
                default:
                    break;
            }
        } catch (Exception e) {
            log.error("消息处理失败", e);
        }

        return Action.CommitMessage;
    }
}
